#Josef Laimer, University of Salzburg, 2010-2016
import chimera
from chimera.baseDialog import ModelessDialog
from chimera import dialogs

import os

import Tkinter
from Tkinter import *
import Pmw
from Pmw import ComboBox
from Pmw import NoteBook
from Pmw import ScrolledFrame
from Pmw import Counter
from Pmw import EntryField

import tkFileDialog
import tkMessageBox
import Tkconstants

from BioDApps_AccSurfSelectionWeb import *
from symbol import try_stmt


DEFAULT_PLOT_WIDTH = 575
DEFAULT_PLOT_HEIGHT = 250

DEFAULT_EXPOSED_REL = 0.75
DEFAULT_EXPOSED_ABS = 100

DEFAULT_MODE = 1
MUTANT_MODE = 2

CONFIG_SURFACE_EXE_NAME = 'calcSurface'

DEFAULT_RUN_TYPE = 1
CONFIG_RUN_TYPE = 2

class AccSurfDialog (ModelessDialog):
    name = 'Accessible Surface Selection Web'   
    title = 'Accessible Surface Selection Web'
    buttons = ('Select', 'Close')   
     
    def __init__(self, mode=DEFAULT_MODE, caller=None, molOsl=None, callback=None):      
        self.callback = callback
        if self.callback != None:
            self.callback(1);
        self.runType = DEFAULT_RUN_TYPE
        self.mode = mode
        self.caller = caller
        
        

        self.models = {}
        self.modelNames = {}
        self.molecules = chimera.openModels.list(modelTypes=[chimera.Molecule])
        
        self.selModel = None
        self.selSubModel = None
        self.showSubModel = False
        
        self.selOSL = molOsl
        
        self.probeSize = 1.4
        
        self.scoreType = IntVar()
        
        self.exposedValueRel = DEFAULT_EXPOSED_REL
        self.exposedValueAbs = DEFAULT_EXPOSED_ABS
        
        self.exposedValue = StringVar()
        
        ModelessDialog.__init__(self, resizable=False)      
        self._closeModelHandlerID = chimera.openModels.addRemoveHandler(self._changeModelHandler, None)
        self._addModelHandlerID = chimera.openModels.addAddHandler(self._changeModelHandler, None)
        
        #print '****', self._closeModelHandlerID, self._addModelHandlerID
        
        if self.runType == DEFAULT_RUN_TYPE:
            self.__fillModels()
            self.__fillModelsCB()
            
            if self.mode == MUTANT_MODE:
                self.buttonWidgets['Select'].config(state = DISABLED)
                self.__drawButtonFrame()
                self.__fillResult()
        
    
    def Close(self):
        try:
            if self._closeModelHandlerID != None:
                chimera.openModels.deleteRemoveHandler(self._closeModelHandlerID)
                self._closeModelHandlerID = None
            if self._addModelHandlerID != None:
                chimera.openModels.deleteAddHandler(self._addModelHandlerID)
                self._addModelHandlerID = None
        except:
            pass
        self.destroy()  


    def Select(self):
        selString = ''
        
        asa = []
        asaValues = self.asaHandler.getValues(self.selOSL, self.probeSize)
        for i in range(0, len(asaValues["residues"])):
            asa.append((i, asaValues["residues"][i], asaValues["asa"][i], asaValues["rasa"][i]))
                    
        i = 2
        if self.scoreType.get() == 1: 
            i = 3
        elif self.scoreType.get() == 2:
            i = 2
        
        for res in asa:
            if res[i] > float(self.exposedValue.get()):
                selString += self.selOSL + ":" + res[1] + " "
        
        oslSel = chimera.selection.OSLSelection(selString)
        oslSel = oslSel.residues()
        itemSel = chimera.selection.ItemizedSelection()
        itemSel.add(oslSel)
        chimera.tkgui.selectionOperation(itemSel)     


    #**********************************************************************************
    #eventhandler
    #**********************************************************************************
    
    #outside trigger
    #===============
    def _changeModelHandler(self,trigger, additional, molList):
        self.molecules = chimera.openModels.list(modelTypes=[chimera.Molecule])
        self.__fillModels()
        self.__fillModelsCB()
    
        
    #inside trigger
    #==============    
    def __selectModel(self, entry):
        if entry == None:
            return
        
        if entry != self.selModel:
            self.selModel = entry
            self.selOSL = "#" + str(self.modelNames[self.selModel])
            submodels = self.models[self.modelNames[self.selModel]]['submodels'].keys()
            if len(submodels) > 1:
                submodels.sort()
                if not self.showSubModel: 
                    self.lSubModel = Label(self.fOptions, text='Submodel')
                    self.cbSubModels = ComboBox(self.fOptions, listbox_width=10, \
                                                dropdown=1, entry_width=10, \
                                                selectioncommand=self.__selectSubmodel, \
                                                scrolledlist_items=[])
                    self.lSubModel.grid(row=0, column=2, sticky=W, padx=5, pady=5)
                    self.cbSubModels.grid(row=0, column=3, sticky=W, padx=5, pady=5)
                
                self.showSubModel = True
                self.cbSubModels.component('scrolledlist').setlist(submodels)
                self.cbSubModels.selectitem(0)
                
                self.selOSL += "." +  str(submodels[0])
            else:
                if self.showSubModel: 
                    self.lSubModel.destroy()
                    self.cbSubModels.destroy()
                    
                self.showSubModel = False
            
            self.selSubModel = submodels[0]
            
            self.__fillResult()
    
    def __selectSubmodel(self, entry):
        self.selSubModel = entry
        self.selOSL = "#" + str(self.modelNames[self.selModel]) + "." + str(entry) 
        self.__fillResult()
        
    def __onCanvasMove(self, event):
        pass
    
    def __canvasMotion(self, event):
        obj = event.widget.find_withtag(CURRENT)
        
        self.cPlot.delete(self.canvasLegend)
        
        if len(obj) < 1:# and not self.canvasLegend == None:
            return
        
        if self.lineOSLs.has_key(obj[0]):
            self.canvasLegend = self.cPlot.create_text(event.x, event.y, text=self.lineOSLs[obj[0]], anchor=E, fill='blue')
    
    
    def __moveExposedByEntry(self, event):
        value = float(self.exposedValue.get())
        if self.scoreType.get() == 1: 
            self.exposedValueRel = value
        elif self.scoreType.get() == 2: 
            self.exposedValueAbs = value
        y = self.y1 - value * self.factorY
        
        if y > 0 and y < DEFAULT_PLOT_HEIGHT: 
            diffY = y -self.cPlot.coords(self.lExposed)[1]
        
            self.cPlot.move(self.lExposed, 0, diffY)
            self.cPlot.move(self.winExposed, 0, diffY)
            self.eExposed.configure(fg='black')
            self.__drawRedTops()
        else: 
            self.eExposed.configure(fg='red')
            
            
    def __scoreTypeChange(self, event=None):
        self.__fillResult()
    
    
    def __probeSizeChange(self):
        if self.probeSize != self.coProbe.get():
            self.probeSize = self.coProbe.get()
            self.__fillResult()
        
    
    #**********************************************************************************
    #helper
    #**********************************************************************************
    
    def __fillModels (self):
        self.models = {}
        self.modelNames = {}
        for m in self.molecules:
            if not self.models.has_key(m.id):
                self.models[m.id] = {}
                self.models[m.id]['submodels'] = {}
                self.models[m.id]['name'] = m.name
                self.models[m.id]['longName'] = '#' + str(m.id) + ' - ' + m.name
                self.modelNames[self.models[m.id]['longName']] = m.id
                
            self.models[m.id]['submodels'][m.subid] = m           
            

    def __fillModelsCB (self):
        if not self.mode == DEFAULT_MODE: 
            return 
        sortModelNames = []
        modelIds = self.models.keys()
        modelIds.sort()
        for k in modelIds:
            sortModelNames.append(self.models[k]['longName'])

        self.cbModels.component('scrolledlist').setlist(sortModelNames)
        
        #print "@1 ", self.selModel
        #if sortModelNames.count(self.selModel) > 0: 
        #    self.cbModels.selectitem(self.selModel)
        if len(sortModelNames) > 0:
            self.cbModels.selectitem(0)
            self.__selectModel(sortModelNames[0])
            
    def __fillResult (self):
        self.cPlot.delete(ALL)
        self.lineOSLs = {}

        asa = []
        maxY = 1.0
        
        asaValues = self.asaHandler.getValues(self.selOSL, self.probeSize)
        for i in range(0, len(asaValues["residues"])):
            asa.append((i, asaValues["residues"][i], asaValues["asa"][i], asaValues["rasa"][i]))
        i = 0
        if self.scoreType.get() == 1: #relative
            maxY = asaValues["maxRAsa"]
            if maxY < self.exposedValueRel:
                self.exposedValueRel = maxY
            i = 3
        elif self.scoreType.get() == 2: #absolute
            maxY = asaValues["maxAsa"]
            if maxY < self.exposedValueAbs:
                self.exposedValueAbs = maxY
            i = 2
            
        maxX = len(asa)
        
        factors = self.__drawAxes(maxX, maxY)
        self.factorX = factors[0]
        self.factorY = factors[1]
        self.__drawScores(factors[0], factors[1], i, asa)
        self.__drawExposedLine(factors[1])
        
    
    def __drawAxes (self, maxX, maxY):
        self.x0=50
        self.x1=DEFAULT_PLOT_WIDTH - 10
        
        self.y0=10
        self.y1=DEFAULT_PLOT_HEIGHT - 20
        
        factorX = float(self.x1-self.x0) / maxX
        factorY = float(self.y1-self.y0) / maxY
        
        self.cPlot.create_line(self.x0-2, self.y1, self.x1, self.y1)
        self.cPlot.create_line(self.x0, self.y0, self.x0, self.y1+2)
        
        self.cPlot.create_line(self.x0-2, self.y0, self.x0+2, self.y0)
        self.cPlot.create_line(self.x1, self.y1-2, self.x1, self.y1+2)
        
        self.cPlot.create_text(self.x0, self.y0, text=('%5.2f' % maxY), anchor=E)
        self.cPlot.create_text(self.x0, self.y1, text='0', anchor=E)
        self.cPlot.create_text(self.x1, self.y1+20, text=str(maxX), anchor=S)
        
        return [factorX, factorY]

    def __drawScores(self, factorX, factorY, i, asavalues):
        j = 0
        asahash = {} 
        for r in asavalues:
            asahash[r[1]] = r[i]
            value = r[i]
            j += 1
            
            x = self.x0 + j * factorX
            y = self.y1 - value * factorY
            
            line = self.cPlot.create_line(x, y, x, self.y1)
            self.lineOSLs[line] = r[1]
        #print asahash
        
    def __drawExposedLine(self, factorY):
        y = self.y1
        if self.scoreType.get() == 1:
            y = self.y1 - self.exposedValueRel * factorY
            self.exposedValue.set(str(self.exposedValueRel))
        elif self.scoreType.get() == 2:
            y = self.y1 - self.exposedValueAbs * factorY
            self.exposedValue.set(str(self.exposedValueAbs))
            
        self.lExposed = self.cPlot.create_line(1, y, DEFAULT_PLOT_WIDTH - 1, y, fill='red')        
        self.eExposed = Entry(self.cPlot, textvariable=self.exposedValue, width=5, state=NORMAL, relief=FLAT, bg='white')
        self.eExposed.bind('<Return>', self.__moveExposedByEntry)

        self.winExposed = self.cPlot.create_window(1, y, window=self.eExposed, anchor=W)
        self.redTops = []
        self.__drawRedTops()
    
    def __drawRedTops(self):
        for rt in self.redTops:
            try: 
                self.cPlot.delete(rt)
            except:
                pass

        asa = []
        asaValues = self.asaHandler.getValues(self.selOSL, self.probeSize)
        for i in range(0, len(asaValues["residues"])):
            asa.append((i, asaValues["residues"][i], asaValues["asa"][i], asaValues["rasa"][i]))
                    
        i = 2
        if self.scoreType.get() == 1: 
            i = 3
        elif self.scoreType.get() == 2:
            i = 2
            
        value = float(self.exposedValue.get())
        yline = self.y1 - value * self.factorY 
        
        j = 0
        for row in asa:
            j += 1
            if row[i] >= value:
                y = self.y1 - row[i]*self.factorY
                x = self.x0 + j * self.factorX
                line = self.cPlot.create_line(x, y, x, yline, fill='red')
                self.redTops.append(line)
                self.lineOSLs[line] = row[2]
        
    def  __returnMask (self, event=None):
        selString = ''
        
        asa = []
        asaValues = self.asaHandler.getValues(self.selOSL, self.probeSize)
        for i in range(0, len(asaValues["residues"])):
            asa.append((i, asaValues["residues"][i], asaValues["asa"][i], asaValues["rasa"][i]))
                    
        i = 2
        if self.scoreType.get() == 1: 
            i = 3
        elif self.scoreType.get() == 2:
            i = 2
        
        for res in asa:
            if res[i] > float(self.exposedValue.get()):
                selString += self.selOSL + ":" + res[1] + " "
        
        oslSel = chimera.selection.OSLSelection(selString)
        self.caller.addASAselection(oslSel.residues(), self.selOSL)
            
    #**********************************************************************************
    #draw GUI
    #**********************************************************************************
    
    #draw option frame
    def __drawOptionFrame (self):
        lModel = Label(self.fOptions, text='Model')    
        self.cbModels = ComboBox(self.fOptions, listbox_width=25, dropdown=1, entry_width=25, \
                                 selectioncommand=self.__selectModel, scrolledlist_items=[])
        lModel.grid(row=0, column=0, sticky=W, padx=0, pady=5)
        
        if self.mode == DEFAULT_MODE:
            self.cbModels.grid(row=0, column=1, sticky=W, padx=5, pady=5)
        else:
            self.lModel = Label(self.fOptions, text=self.selOSL)
            self.lModel.grid(row=0, column=1, sticky=W, padx=5, pady=5)
            
        Label(self.fOptions, text="\t\t\t\t ").grid(row=0, column=2, columnspan=2, sticky=W, padx=5, pady=5) #placeholder
        
        
        lProbe = Label(self.fOptions, text='Probe size')
        lProbe.grid(row=1, column=0, sticky=W, padx=0, pady=5)
        self.coProbe = Counter(self.fOptions, entryfield_value='1.4', entry_width=5, datatype={'counter': 'real', 'separator': '.'}, \
                          entryfield_validate={'validator': 'real', 'min':'0.1', 'max':'2.5','separator':'.'}, \
                          increment= .1, entryfield_modifiedcommand=self.__probeSizeChange)
        
        self.coProbe.grid(row=1, column=1, sticky=W, padx=5, pady=5)
        
    
    #draw result frame
    def __drawResultFrame (self):
        fScoreType = Frame(self.fResults)
        fScoreType.pack(side=TOP, fill=X, expand=NO)
        
        Label(fScoreType, text='Score type: ').pack(side=LEFT)
        Radiobutton(fScoreType, text='relative', value=1, variable=self.scoreType, command=self.__scoreTypeChange).pack(side=LEFT)
        Radiobutton(fScoreType, text='absolute', value=2, variable=self.scoreType, command=self.__scoreTypeChange).pack(side=LEFT)
        self.scoreType.set(1)
        
        fResultPlot = Frame(self.fResults)
        fResultPlot.pack(side=TOP, fill=BOTH, expand=YES)
        
        self.cPlot = Canvas(fResultPlot, width=DEFAULT_PLOT_WIDTH, height=DEFAULT_PLOT_HEIGHT, bg='gray')
        self.cPlot.pack(side=LEFT, anchor='w', expand=YES)
        
        self.cPlot.bind('<Motion>', self.__canvasMotion)
        
        self.canvasLegend = None
        
    def __drawButtonFrame (self):
        self.btnReMask = Button(self.fButtons, text='Return Mask', command=self.__returnMask, state=NORMAL)
        self.btnReMask.pack(side=RIGHT, anchor='w', expand=NO)
    
    def __selectCalcSurface(self, event=None):
        """
        Event: select calc-surface executeable
        """
        path = tkFileDialog.askopenfilename(title="choose 'calc-surface' executable", initialfile="calc-surface")
        if path != ('') and  os.path.exists(path):
            self.calcSurfaceVar.set(path)
            
    def __saveCalcSurface(self):
        if not os.path.exists(self.calcSurfaceVar.get()):
            tkMessageBox.showerror("Error", "%s does not exists!" % self.calcSurfaceVar.get())
        else:
            f = file(os.path.join(os.environ.get('HOME'), '.chimeraProsaMod.conf'), 'a')
            f.write(CONFIG_SURFACE_EXE_NAME + ':' + self.calcSurfaceVar.get() + '\n')
            f.close()
            self.runtype = DEFAULT_RUN_TYPE
            self.fMain.destroy()
            self.fillInUI(self.parent)       

    #init GUI
    #========
    def fillInUI(self, parent):

        self.asaHandler = ASAHandler()
        self.buttonWidgets["Select"].config(state=Tkconstants.NORMAL)
        self.parent = parent
        
        self.fMain = Frame(self.parent, bg="#d3ebf7")
        self.fOptions = Frame(self.fMain, bg="#d3ebf7")
        self.fResults = Frame(self.fMain, bg="#d3ebf7")
        self.fButtons = Frame(self.fMain, bg="#d3ebf7")
        
        self.fMain.pack(fill=BOTH, expand=YES)
        self.fOptions.pack(side=TOP, fill=X, expand=NO)
        self.fResults.pack(side=TOP, fill=BOTH, expand=NO)
        self.fButtons.pack(side=TOP, fill=X, expand=NO)
    
        self.__drawOptionFrame()
        self.__drawResultFrame()
        
        if self.callback != None:
            self.callback(0);
        
chimera.dialogs.register(AccSurfDialog.name, AccSurfDialog)

