import os
from chimera import preferences

class Config:
    """
    this class loads a config file 
    for now it only reads the url and contains the used AAList
    but further statics can easily be added 
    this config file is for persistant and esys acces to statics like the urls
    """
    ########## SINGLETON ##########
    __instance = None
    
    @classmethod
    def getInstance(cls):
        """
        necessary for the singleton pattern
        makes a new instance if there isnt an instance otherwise it retrns the instance
        """
        if cls.__instance == None:
            cls.__instance = cls()
        return cls.__instance
    
    def __init__(self):
        """
        necessary for the singleton pattern
        normal Contructor"
        """
        self.lPath = os.path.join(os.path.dirname(preferences.preferences._filename), "config.txt")
        self.url = 'https://biwww.che.sbg.ac.at/biodapps/pl/rest/%s'
        self.loadFile()
               
    def reset(self):
        """
        necessary for the singleton pattern
        resets the id of the singleton
        """
        self.lastId = 0
        
    def getNextId (self):
        """
        necessary for the singleton pattern
        returns the next id of the singleton
        """
        self.lastId += 1
        return self.lastId
    
    def getId (self):
        """
        necessary for the singleton pattern
        returns the last id of the singleton
        """
        return self.lastId
    ########## SINGLETON ########## 

    def loadFile(self):
        """
        sets a default value for the url 
        and overwrites it if file exists with the value of the config file in the format
        url:http://chimera2016/....
        """
        self.url = 'https://biwww.che.sbg.ac.at/biodapps/pl/rest/%s'
        if (os.path.exists(self.lPath)):
            rconfig = open(self.lPath, 'r')
            for line in rconfig:
                line.rstrip()
                entry = line.split(':',1) # only spilts on the first ":"
                self.url = entry[1]   
            rconfig.close()
          
    
    def setUrl(self,url):
        """
        @param url: url of the server 
        setter of the url
        """
        self.url = url
    
    def getUrl(self):
        """
        getter of the url
        @return: self.url: used url of server
        @type: self.url: str
        """
        return self.url
    
    def getAaList(self):
        """
        getter of the used AA set 
        @return: list of the used AAs 
        @type: list<str>
        """
        return [
           "A",
           "R",
           "N",
           "D",
           "C",
           "E",
           "Q",
           "G",
           "H",
           "I",
           "L",
           "K",
           "M",
           "F",
           "P",
           "S",
           "T",
           "W",
           "Y",
           "V",
           "X",
           "HYDROPHOBIC",
           "POLAR",
           "AROMATIC",
           "ALIPHATIC",
           "CHARGED",
           "POSITIVE",
           "NEGATIVE",
           "SULPHURCONT",
           "TINY",
           "SMALL",
           "LARGE"
           ]
        
        
        
