from Tkinter import *           # @UnusedWildImport
import tkMessageBox

from threading import Thread
import Pmw
from Pmw import ComboBox        # @UnresolvedImport
from chimera.baseDialog import ModelessDialog
import chimera
from chimera import selection
from __init__ import validateMaestro, AAMapping, DataProviderMaestro
import Tkconstants as tkc
from StringIO import StringIO   
import PIL                      # @UnresolvedImport
import Midas              
import BioDApps_Core.gui    
from BioDApps_Core import Connector, requests
from BioDApps_Core import Config as BConfig
import ttk
import BioDApps_Core.multiListbox as mlb



class MaestroDialog(ModelessDialog):
    """
    Generates and fills GUI for the input of the user to send a job
    The GUI has 3 different Options to start a job
    The first one can send a job for a sensitivity profile
    The Second one shows a new part of the GUI to create a List of residues
    The 3rds is the same as the 2nd one but with more options 
        and you can scan for (de)stabilizing mutations
    You can send a Job to the Maestro Server to get the result
    
    inherited from ModelessDialog  
    """
    
    title = "Maestro"
    name = "Maestro"
    buttons = ('Close', 'Apply')
    
    def __init__(self, **kw):
        """
        Constructor
        
        Special:
        addRemoveHandler/addAddHandler (Observer to get notified if there is a new molecule
        @param **kw: is needed for super()  
        """        
        self.frame2 = None
        self.v = StringVar()
        self.optionElements = []
        
        # fill in GUI:
        self.nameList = []
        ModelessDialog.__init__(self, **kw)
        
        self._closeModelHandlerID = chimera.openModels.addRemoveHandler(self._changeModelHandler, None)
        self._addModelHandlerID = chimera.openModels.addAddHandler(self._changeModelHandler, None)
    
    # fill in GUI: (model change event handler)
    def _changeModelHandler(self, trigger, additional, molList):
        """
        If there is a new model or a model is deleted than update the combo boxes 
        to show the right molecules and the right combination of the different data (id,subid,chain)
        """
        
        self.loadStruct()  # refill combo box
        
        
        self.chimeraUpdateComboBox()
        
            
        
    # what happend when press Close  
    def Close(self):
        """
        To close the GUI
        And close the Remove- and AddHandler 
        """  
        if self._closeModelHandlerID != None:
            chimera.openModels.deleteRemoveHandler(self._closeModelHandlerID)
            self._closeModelHandlerID = None
        if self._addModelHandlerID != None:
            chimera.openModels.deleteAddHandler(self._addModelHandlerID)
            self._addModelHandlerID = None
        
        ModelessDialog.Close(self)
        
        self.destroy()
        
    def Apply(self):
        """
        Starts a new thread for the apply logic (is in doApply) if validation is successfully 
        disables apply button so the GUI can't send any further job until this one is completed
        already creates resultGUI
        gets automatically called when Apply button is pressed
        """
        validationInfo = validateMaestro(self)
        if(validationInfo[0]):
            self.buttonWidgets['Apply'].config(state=DISABLED)
            
            if (self.choice.get() == 1): 
                self.r  = MResultSensiProf()
            elif(self.choice.get() == 2):
                self.r = MaestroTable(self.structures)
            elif(self.choice.get() == 3):
                self.r = MSResult(self.structures)
            t  = Thread(target = self.doApply)
            t.start()
        else:
            tkMessageBox.showerror("Error", validationInfo[1])
    
    
                
                
    def doApply(self):
        """
        Starts the job and sends the data of the User to the servers of maestro.
        Validates the input of the user.
        It also fills the resultGUI to show the results
        """
        
        #create empty dataProvider and fill it
        dataProvider = DataProviderMaestro()
        dataProvider.readGui(self)
        dataProvider.setPdbs(self.getPdbListMaestro())
        
        #create BioDAppsConnector
        BioDAppsC = Connector.MaestroConnector(dataProvider, self.r.updateProgress)
        
        
        try:
            result = BioDAppsC.startMaestroJob()
            if (self.choice.get() == 1):  
                result = "https://biwww.che.sbg.ac.at/biodapps/pl/rest" + result['result']['downloadPng']       
            print result
            self.r.setResult(result)

        except Exception as e:
            tkMessageBox.showerror("Error", "Service not available")
            print e
        
        self.buttonWidgets['Apply'].config(state=NORMAL)    
        
        
    def loadStruct(self):
        """
        Creates the dictionary for each molecule that has been loaded into chimera 
        dictionary has Molecule-names as keys
        values are dictionaries again, those have 5 keys:
            id -> of molecule
            submodels -> needed for MAESTRO
            chains -> value: map (keys chainID ; values list of residues | key '*' predefines list for all residues)
            ptrModel -> access to whole model (from here you can get any information you want)
            chainId -> access to the residues
            
            example:
            {u'1crn': { 'ptrModel': <_molecule.Molecule object at 0x000000000E9A6DF0>,
                   'chains': {u'A': ['T1.A', 'T2.A', ... ], '*': ['T1.A', 'T2.A', ...]},
                   'id': 0,
                   'submodels': [0]
                  },
            u'1yti': {'ptrModel': <_molecule.Molecule object at 0x000000000E9E4120>,
                  'chains': {u'A': ['P1.A', 'Q2.A', ...], u'I': ['F101.I', ...], '*': ['P1.A ...]},
                  'id': 1,
                  'submodels': [0]
                  }
        }
            
            
        """
        self.curModels = chimera.openModels.list(modelTypes=[chimera.Molecule])  # current opened Molecule
        
        self.structures = {}
        for m in self.curModels:  # Loop to get all Molecule-names  
            if not self.structures.has_key(m.name):
                
                self.structures[m.name] = {}                #create empty map, following adds content to it
                self.structures[m.name]['id'] = m.id
                self.structures[m.name]['submodels'] = []   #contains all submodels, is added later
                self.structures[m.name]['chains'] = {}      #each residue belongs to a chain, key is chain value is list of residues
                self.structures[m.name]['chains']['*'] = [] #means all chains (points to a list filled with all residues)
                self.structures[m.name]['ptrModel'] = m     # Pointer to model to get the pdbstring easier
                # fill chains in map
                residues = m.residues
                for r in residues:
                    if (r.id.chainId not in self.structures[m.name]['chains']): #if this chain is not included in map yet, add it
                        self.structures[m.name]['chains'][r.id.chainId] = []
                    # first create residue's name before inserting
                    aaMap = AAMapping();
                    aaLetter = aaMap.aaThreeToOne(str(r.type))
                    residueName = str(aaLetter) + str(r.id.position) + "." + str(r.id.chainId)
                    self.structures[m.name]['chains'][r.id.chainId].append(residueName)
                    self.structures[m.name]['chains']['*'].append(residueName)
            # fill submodels in map
            self.structures[m.name]['submodels'].append(m.subid)
        
        
    # When PDBID-field gets changed, this fkt updates Substr, chains and mutate form
    def chimeraUpdateComboBox(self):
        """
        Updates  comboboxes according to the user selection.
        Updates the ID, SubID, Chain and Mutate_from.
        called by _changeModelHandler
        @attention: if user selects sth in Chain_Menu, the mutation residue box is cleared 
        """
        
        self.currentSel = ''
        self.currentSel = self.PDB_ID_Menu.get()
            
        self.PDB_IDs = self.structures.keys()
        self.PDB_ID_Menu.component('scrolledlist').setlist(self.PDB_IDs)
        
        # lock selection menu from writing
        self.PDB_ID_Menu.component("entryfield").component("entry").configure(state=DISABLED)
        
        # If the current seleection was deleted (not in the scrolled list) than set to empty string
        if (self.currentSel not in self.PDB_ID_Menu.component('scrolledlist').get()):
            self.currentSel = ''
        
        if (len(self.PDB_IDs) > 0):           
            if (self.currentSel == ''):
                self.PDB_ID_Menu.selectitem(str(self.PDB_IDs[0]))
            else:            
                self.PDB_ID_Menu.selectitem(str(self.currentSel))
            self.PdbIdUpdateComboBox()
            self.chainUpdateComboBox()
            self.enableGUI()  # default is True
        else:
            self.PDB_ID_Menu.component('entryfield').clear()
            self.Substructure_Menu.component('entryfield').clear()
            self.Chain_Menu.component('entryfield').clear()
        
            for element in self.optionElements:
                element.destroy() 
            self.optionElements = []
            if self.frame2 is not None: 
                self.frame2.destroy()
                
            self.enableGUI(False)  # hides Comboboxes and disables Apply button            

        
    # When PDBID-field gets changed, this fkt updates Substr, chains and mutate form
    def PdbIdUpdateComboBox_(self, entry):  
        """
        Necessary because of the requested parameter entry
        (this function is called automatically) 
        """      
        self.PdbIdUpdateComboBox()
        
    def PdbIdUpdateComboBox(self): 
        """
        Updates all Comboboxes if the PDB_ID was changed
        is called by: PdbIdUpdateComboBox_ and fillInUI
        """
        self.Chain_Menu.selectitem('*')     

        self.Substructures = self.structures[self.PDB_ID_Menu.get()]['submodels']
        self.Substructure_Menu.component('scrolledlist').setlist(self.Substructures)
        self.Substructure_Menu.selectitem(self.Substructures[0])


        # look selection menu for writing
        if (len(self.PDB_IDs) > 0):
            self.Substructure_Menu.component("entryfield").component("entry").configure(state=DISABLED)
        else:
            self.Substructure_Menu.component('entryfield').clear()
            
        self.Chains = self.structures[self.PDB_ID_Menu.get()]['chains']
        if (len(self.PDB_IDs) > 0):
            self.Chain_Menu.component('scrolledlist').setlist(self.Chains)
        else:
            self.Chain_Menu.component('entryfield').clear()
        # look selection menu for writing
        self.Chain_Menu.component("entryfield").component("entry").configure(state=DISABLED)
        
        if (len(self.PDB_IDs) == 1):
                self.fillOptionFrame()
        
        self.updateSecondPartOfGui()
        
        
        
    def updateSecondPartOfGui(self):
        """
        updates the mutate_from List with all concerning data (ID,DubId,Chain,Residues)
        if a PDB-ID was changed (radiobutton2 or radiobutton3)
        
        """
        
        if (len(self.PDB_IDs) > 0):
            
            # fixes the bug if you check the second radiobutton before loading a structure
            # loads the second GUI after loading in a protein but if you have 1 Protein after the loading prcess
            
            
            if(self.choice.get() == 2):  
                self.Mutate_from = self.structures[self.PDB_ID_Menu.get()]['chains'][self.Chain_Menu.get()]
                self.Mutate_from_Menu.component('scrolledlist').setlist(self.Mutate_from)
                self.Mutate_from_Menu.selectitem(self.Mutate_from[0])
                # List of selected mutations needs to be cleared
                self.text1.delete(0.0, END)
                
                
            if(self.choice.get() == 3):  
                
                self.Mutate_from = self.structures[self.PDB_ID_Menu.get()]['chains'][self.Chain_Menu.get()]
                self.Mutate_from_Menu.component('scrolledlist').setlist(self.Mutate_from)
                self.Mutate_from_Menu.selectitem(self.Mutate_from[0])
                # List of selected mutations needs to be cleared
                self.text1.delete(0.0, END)
                
    
    def chainUpdateComboBox_(self, entry):
        """
        Necessary because of the requested parameter entry
        (this function is called automatically)
        When chian-field gets changed, this method updates mutate form
        """
        self.chainUpdateComboBox()
        
    def chainUpdateComboBox(self):
        """
        Updates mutate_from combo boxes if the Chain was changed
        Is called from: chainUpdateComboBox_ and chimeraUpdateComboBox
        """
        self.updateSecondPartOfGui()
        
    
    def addMutationToList (self):
        """
        Adds a residue to the ResidueJobList (if Button Add to List is pressed)
        gets information from Mutate_from_Menu and Mutate_to_Menu
        """
        self.text1.insert(END, self.Mutate_from_Menu.get() + "{" + self.Mutate_to_Menu.get() + "}\n")
        self.text1.see(END)
      
    # is called when importSelection button is pressed  
    def importSelection(self):
        """
        imports all selected residues from chimera into the ResidueJobList and change the Residue from 3-Letter Code to 1-Letter Code
        selection (selRes = chimera.selection.currentResidues()) is displayed as error, but works
        """
        selRes = selection.currentResidues()
        self.importResidues(selRes)
          
    
    def importResidues(self, resList):    
        """
        Subfunction to execute a importSelection
        validates if the selected residue of in the right protein and if the 
        residue is valid
        @param resList:  list of residues
        """
        for k in resList:
            mutate = ""  # string
            # change k.type (aaLetter)
            aaMap = AAMapping()
            aaLetter = str(aaMap.aaThreeToOne(k.type))  # get_aa_letter(k.type)
            
            residueName = aaLetter + str(k.id.position) + "." + str(k.id.chainId)   #create residue name
            #check if residueName is in residueComboboox -> this avoids from selecting a residue of a pdb which is not the current selected (in gui)
            if residueName in self.Mutate_from:
                
                mutTo = self.Mutate_to_Menu.get()   #get AA to which this residue shall mutate
                if mutTo is not aaLetter:           #must not mutate to it self
                    mutate = residueName +"{" + mutTo + "}\n"   #create final string which is going to be added to text box
                    self.text1.insert(END, mutate)              #insert into text box
                    self.text1.see(END)                         #refresh
    
    def openMaskbuilder(self):
        """
        opens the Maskbuilder to import residues with the maskbuilder
        """
        # osl == molecule osl id
        
        # get current selection
        self.protname = self.PDB_ID_Menu.get()
        idOSL = self.structures[self.protname]['id']
        osl = str("#" + str(idOSL))
        self.maskBuilder = BioDApps_MaskBuilder.gui.MaskBuilderDialog(BioDApps_MaskBuilder.gui.MUTANT_MODE, osl, self)
          
        
        
    def addMask(self, residues, molOSL):    
        """
        @param residues: selected residues of the maskbuilder
        @param molOSL: molOSL OSL selection of the maskbuilder (not used)
        callback function for Maskbuilder
        imports the selected residues of the Maskbuilder (fills the mutate_joblist)
        """   
        self.importResidues(residues)
        
        
    # Evaluate specific mutations addon
    def fillOptionFrame(self):
        """
        Creates and shows the second or third part of the GUI if needed.
        If the second radio Button is selected a secondGUIpart appears to work with the residue(list)
        You can make a list for mutation and change the pH-Value
        If the Third Radio Button is selected you can fill in the same 
        plus you can choose between stabilizing an desstabilizing and you can use an ASA FIlter 
        and you can choose an Algorithm for calculation
        some Comboboxes have been locked(no writing allowed, only selecting one of the given values) so that user can't insert any invalid values:
            self.Mutate_from_Menu.component("entryfield").component("entry").configure(state=DISABLED)
            and other
            this is why font color is gray in GUI
        """
        if (self.choice.get() == 2 and len(self.PDB_IDs) > 0):  # creat when needed
            
            for element in self.optionElements :
                element.destroy() 
            self.optionElements = []
            
            if (self.frame2 != None):
                self.frame2.destroy()
            
            self.frame2 = Frame(self.parent, bg="#d3ebf7")
            self.frame2.pack(padx=0, pady=0, expand=YES, fill=BOTH)
            self.subframe2 = Frame(self.frame2, bg="#d3ebf7")
            self.subframe2.pack(padx=0, pady=0, expand=YES, fill=BOTH)
            
            self.optionElements.append(self.subframe2)  # list which will destroyed
            
            # formations box
            self.optionlabelframe1 = LabelFrame(self.subframe2, text="Select specific mutations", bg="#d3ebf7")
            self.optionlabelframe1.pack(expand="yes", padx=0, pady=5)
            
            # labels
            self.optionlabel1 = Label(self.optionlabelframe1, text="Mutate           ", bg="#d3ebf7").grid(row=0, column=0)
            self.optionlabel2 = Label(self.optionlabelframe1, text="List of Mutations", bg="#d3ebf7").grid(row=1, column=0)
            
            # list for the Mutate_from dropdown
            if (len(self.PDB_IDs) > 0):
                self.Mutate_from = self.structures[self.PDB_ID_Menu.get()]['chains'][self.Chain_Menu.get()]
            
            # Mutate_from dropdown
            self.Mutate_from_Menu = ComboBox(self.optionlabelframe1, listbox_width=30, dropdown=1, entry_width=25, scrolledlist_items=self.Mutate_from)
            self.Mutate_from_Menu.grid(row=0, column=1)
            self.Mutate_from_Menu.selectitem(self.Mutate_from[0])
            # look selection menau for writing
            self.Mutate_from_Menu.component("entryfield").component("entry").configure(state=DISABLED)
        
            self.optionlabel3 = Label(self.optionlabelframe1, text="to", bg="#d3ebf7").grid(row=0, column=2)
            
            # list for the Mutate_to dropdown
            self.Mutate_to = BConfig.Config.getInstance().getAaList()

            # Mutate_to dropdown
            self.Mutate_to_Menu = ComboBox(self.optionlabelframe1, listbox_width=30, dropdown=1, entry_width=25, scrolledlist_items=self.Mutate_to)
            self.Mutate_to_Menu.grid(row=0, column=3)
            self.Mutate_to_Menu.selectitem(self.Mutate_to[0])
            # look selection menu for writing
            self.Mutate_to_Menu.component("entryfield").component("entry").configure(state=DISABLED)
            
            #####textbos with Scrollbar
            self.frametext = Frame(self.optionlabelframe1, bg="#d3ebf7")
            self.frametext.grid(row=1, column=1)
            
            self.scrollbar = Scrollbar(self.frametext)
            self.scrollbar.pack(side=RIGHT, fill=Y)

            self.text1 = Text(self.frametext, width=25, height=7, yscrollcommand=self.scrollbar.set)
            self.text1.pack()

            self.scrollbar.config(command=self.text1.yview)
            #####
            
            #####import and add button
            self.buttonframe = Frame(self.optionlabelframe1, bg="#d3ebf7")
            self.buttonframe.grid(row=1, column=3)
            
            self.add_button = Button(self.buttonframe, text="             Add to List             ", command=self.addMutationToList) 
            self.add_button.pack()
            self.import_button = Button(self.buttonframe, text="Import from Chimera selection", command=self.importSelection) 
            self.import_button.pack(padx=0, pady=5)
            self.createMask_button = Button(self.buttonframe, text="             Create mask            ", command=self.openMaskbuilder) 
            self.createMask_button.pack()
            
            # formations box
            self.optionlabelframe2 = LabelFrame(self.subframe2, text="Further options", bg="#d3ebf7")
            self.optionlabelframe2.pack(expand="yes", padx=0, pady=5)
            
            # ph value entry with min max
            self.optionlabel4 = Label(self.optionlabelframe2, text="pH-value", bg="#d3ebf7").grid(row=0, column=0)
            
            ph_val = StringVar(self.optionlabelframe2)
            ph_val.set("7")
            self.ph_value = Spinbox(self.optionlabelframe2, from_=5.5, to=8.5, increment=0.1, textvariable=ph_val)
            self.ph_value.grid(row=0, column=1)
            
            # variable which know whith Radiobutton is selected
            self.mutate_choice = IntVar()
            self.mutate_choice.set(1)
        
            # Radiobuttons
            self.optionrb = Radiobutton(self.optionlabelframe2, text="Independet Mutations", variable=self.mutate_choice, value=1, bg="#d3ebf7").grid(row=1, column=0)
            self.optionrb = Radiobutton(self.optionlabelframe2, text="Combined Mutations  ", variable=self.mutate_choice, value=2, bg="#d3ebf7").grid(row=2, column=0)
        
        
        elif (self.choice.get() == 3 and len(self.PDB_IDs) > 0):  # destroy when not needed
            #### Add Muationsscanpart:
            
            # let second part of GUI disappear
            for element in self.optionElements :
                element.destroy() 
            self.optionElements = []
            if (self.frame2 != None):
                self.frame2.destroy()
            
            self.frame2 = Frame(self.parent, bg="#d3ebf7")  # create second part(frame)
            self.frame2.pack(padx=0, pady=0, expand=YES, fill=BOTH)  # pack second part/frame
            
            
            # ## General: first sub frame
            self.subframe2 = Frame(self.frame2, bg="#d3ebf7")  # create first sub frame of second part
            self.subframe2.pack(padx=0, pady=0, expand=YES, fill=BOTH)  # pack sub frame
            self.optionElements.append(self.subframe2)  # register: add to list whose elements will be destroyed if another radio button is selected
            
            # formations box
            self.optionlabelframe1 = LabelFrame(self.subframe2, text="General", bg="#d3ebf7")
            self.optionlabelframe1.pack(expand="yes", padx=0, pady=5)  # add Label to sub frame
            
            # labels
            self.optionlabel1 = Label(self.optionlabelframe1, text="Select the Type of the experiment:  ", bg="#d3ebf7").grid(row=0, column=0, sticky=W)
            self.optionlabel2 = Label(self.optionlabelframe1, text="Choose an algorithm:                ", bg="#d3ebf7").grid(row=1, column=0, sticky=W)
            self.optionlabel3 = Label(self.optionlabelframe1, text="PH-value:                           ", bg="#d3ebf7").grid(row=2, column=0, sticky=W)
            self.optionlabel4 = Label(self.optionlabelframe1, text="Maximum point mutations:            ", bg="#d3ebf7").grid(row=3, column=0, sticky=W)
            
            # Radio buttons for label1
            # variable that knows which radio button is selected
            self.experimentTye_choice = IntVar()
            self.experimentTye_choice.set(1)
            # Radio buttons
            self.optionrb = Radiobutton(self.optionlabelframe1, text="stabilizing ", variable=self.experimentTye_choice, value=1, bg="#d3ebf7").grid(row=0, column=1, sticky=W)
            self.optionrb = Radiobutton(self.optionlabelframe1, text="destabilizing ", variable=self.experimentTye_choice, value=2, bg="#d3ebf7").grid(row=0, column=2, sticky=W)

            # algorithm drop down for label2
            # list for dropdown
            self.choosableAlgorithms = [
                   "greedy search",
                   "evolutinary algorithm",
                   "SS-Scan"
                   ]
            # algorithm dropdown
            self.choosenAlgorithm = ComboBox(self.optionlabelframe1, entry_width=20, listheight=60, scrolledlist_items=self.choosableAlgorithms)  # Param: entry_width=how long is box(left<>right) 
            self.choosenAlgorithm.grid(row=1, column=1, columnspan=2, sticky=W)
            self.choosenAlgorithm.selectitem(self.choosableAlgorithms[0])
            # lock selection menu from writing
            self.choosenAlgorithm.component("entryfield").component("entry").configure(state=DISABLED)
            
            # ph-value
            ph_val = StringVar(self.optionlabelframe1)
            ph_val.set("7")
            self.ph_value = Spinbox(self.optionlabelframe1, from_=5.5, to=8.5, increment=0.1, width=5, textvariable=ph_val)  # params: width=defalut20 
            self.ph_value.grid(row=2, column=1, sticky=W)
            
            # algorithm drop down for label3 Maximum point mutations
            # list for dropdown
            self.MaximumPointsList = [
                   "1",
                   "2",
                   "3",
                   "4",
                   "5"
                   ]
            # algorithm dropdown
            self.MaximumPoints = ComboBox(self.optionlabelframe1, entry_width=20, listheight=95, scrolledlist_items=self.MaximumPointsList)  # Param: entry_width=how long is box(left<>right) 
            self.MaximumPoints.grid(row=3, column=1, columnspan=2, sticky=W)
            self.MaximumPoints.selectitem(self.MaximumPointsList[2])
            # lock selection menu from writing
            self.MaximumPoints.component("entryfield").component("entry").configure(state=DISABLED)
            
            
            # ## Residue restriction: second sub frame
            self.subframe3 = Frame(self.frame2, bg="#d3ebf7")  # create second sub frame of second part
            self.subframe3.pack(padx=0, pady=0, expand=YES, fill=BOTH)  # pack sub frame
            self.optionElements.append(self.subframe3)  # register: add to list whose elements will be destroyed if another radio button is selected
            
            # formations box
            self.optionlabelframe2 = LabelFrame(self.subframe3, text="Residue restriction", bg="#d3ebf7")
            self.optionlabelframe2.pack(expand="yes", padx=0, pady=5)  # add Label to sub frame
            
            # labels
            self.optionlabel4 = Label(self.optionlabelframe2, text="Allow residue    ", bg="#d3ebf7").grid(row=0, column=0, sticky=W)
            self.optionlabel5 = Label(self.optionlabelframe2, text="Allowed residues ", bg="#d3ebf7").grid(row=1, column=0, sticky=W)
            
            # list for residues dropdown
            self.Mutate_from = []  # define, will be filled in if
            if (len(self.PDB_IDs) > 0):
                # TODO Update (bugfix von Jonas: Task II.2)
                self.Mutate_from = self.structures[self.PDB_ID_Menu.get()]['chains'][self.Chain_Menu.get()]
            # residues dropdown
            self.Mutate_from_Menu = ComboBox(self.optionlabelframe2, entry_width=25, scrolledlist_items=self.Mutate_from)
            self.Mutate_from_Menu.grid(row=0, column=1)
            self.Mutate_from_Menu.selectitem(self.Mutate_from[0])
            # lock selection menu from writing
            self.Mutate_from_Menu.component("entryfield").component("entry").configure(state=DISABLED)
            
            
            
            # textbos with Scrollbar
            self.frameTextbox = Frame(self.optionlabelframe2, bg="#d3ebf7")
            self.frameTextbox.grid(row=1, column=1)
            
            self.textBoxScrollbar = Scrollbar(self.frameTextbox)
            self.textBoxScrollbar.pack(side=RIGHT, fill=Y)

            self.text1 = Text(self.frameTextbox, width=25, height=7, yscrollcommand=self.textBoxScrollbar.set)
            self.text1.pack()

            self.textBoxScrollbar.config(command=self.text1.yview)
            #
            
                        # list for the Mutate_to dropdown
            self.Mutate_to = BConfig.Config.getInstance().getAaList()
            
            # Mutate_to dropdown
            self.Mutate_to_Menu = ComboBox(self.optionlabelframe2, listbox_width=30, dropdown=1, entry_width=25, scrolledlist_items=self.Mutate_to)
            self.Mutate_to_Menu.grid(row=0, column=3)
            self.Mutate_to_Menu.selectitem(self.Mutate_to[self.Mutate_to.index('X')])   #set default to X
            # look selection menu for writing
            self.Mutate_to_Menu.component("entryfield").component("entry").configure(state=DISABLED)
            
            self.buttonframe = Frame(self.optionlabelframe2, bg="#d3ebf7")
            self.buttonframe.grid(row=1, column=3)
            
            # import and add button
            self.add_button = Button(self.buttonframe, text="             Add to List            ", command=self.addMutationToList) 
            self.add_button.pack()
            self.import_button = Button(self.buttonframe, text="Import from Chimera selection", command=self.importSelection) 
            self.import_button.pack()
            self.createMask_button = Button(self.buttonframe, text="             Create mask            ", command=self.openMaskbuilder) 
            self.createMask_button.pack()
            #
            
            
            # ## Further options: third sub frame
            self.subframe4 = Frame(self.frame2, bg="#d3ebf7")  # create second sub frame of second part
            self.subframe4.pack(padx=0, pady=0, expand=YES, fill=BOTH)  # pack sub frame
            self.optionElements.append(self.subframe4)  # register: add to list whose elements will be destroyed if another radio button is selected
            
            # formations box
            self.optionlabelframe3 = LabelFrame(self.subframe4, text="Further options", bg="#d3ebf7")
            self.optionlabelframe3.pack(expand="yes", padx=0, pady=5)  # add Label to sub frame
            
            # labels
            self.optionlabel6 = Label(self.optionlabelframe3, text="ASA filter: ", bg="#d3ebf7").grid(row=0, column=0, sticky=W)
            self.optionlabel7 = Label(self.optionlabelframe3, text="ASA cutoff: ", bg="#d3ebf7").grid(row=1, column=0, sticky=W)
            self.optionlabel7_a = Label(self.optionlabelframe3, text="%   ", bg="#d3ebf7").grid(row=1, column=2, sticky=W)
            self.optionlabel8 = Label(self.optionlabelframe3, text="Display top mutations: ", bg="#d3ebf7").grid(row=2, column=0, sticky=W)
            
            # ASA filter dropdown
            # list for dropdown
            self.choosableASAs = (
                   'none',  # if "none" is active ASACutoff is disabled, see updateAsaFilterChanged
                   'buried',
                   'exposed'
                   )
            
            # algorithm dropdown
            self.choosenASA = ComboBox(self.optionlabelframe3, entry_width=15, listheight=60, scrolledlist_items=self.choosableASAs, selectioncommand=self.updateAsaFilterChanged)  # Param: entry_width=how long is box(left<>right) 
            self.choosenASA.grid(row=0, column=1, columnspan=3, sticky=W)
            self.choosenASA.selectitem(self.choosableASAs[0])  # ## TODO if NONE selected disable ASA cutoff
            # set function for triggering
            
            # lock selection menu from writing
            self.choosenASA.component("entryfield").component("entry").configure(state=DISABLED)
            
            # ASA cutoff
            self.ASACutoffText = StringVar(self.optionlabelframe3)
            self.ASACutoffText.set("30")
            self.ASACutoff = Spinbox(self.optionlabelframe3, from_=0, to=100, increment=1, width=6, textvariable=self.ASACutoffText, command=self.updateAsaCutoffScale)  # params: width=defalut20 
            self.ASACutoff.grid(row=1, column=1, sticky=W)
            
            # ASA Scale
            self.ASACutoffScale = Scale(self.optionlabelframe3, from_=0, to=100, orient=HORIZONTAL, bg="#d3ebf7", borderwidth=0.0, showvalue=0)  # showvalue=if is 1 value will be shown above slider
            self.ASACutoffScale.bind("<ButtonRelease-1>", self.updateAsaCutoffSpin);  # bind to trigger 
            self.ASACutoffScale.grid(row=1, column=3, sticky=W)
            self.updateAsaCutoffScale()  # do it once so it shows correct value from the beginning
            
            self.updateAsaFilterChanged(self.choosenASA.get())
            
            
            # display top mutations
            self.TopMutationsText = StringVar(self.optionlabelframe3)
            self.TopMutationsText.set("100")
            self.TopMutations = Spinbox(self.optionlabelframe3, from_=1, to=1000, increment=1, width=6, textvariable=self.TopMutationsText, command=self.updateTopMutationsScale)  # params: width=defalut20 
            self.TopMutations.grid(row=2, column=1)
            
            # top mutations Scale
            self.TopMutationsScale = Scale(self.optionlabelframe3, from_=1, to=1000, orient=HORIZONTAL, bg="#d3ebf7", borderwidth=0.0, showvalue=0)  # showvalue=if is 1 value will be shown above slider ||
            self.TopMutationsScale.bind("<ButtonRelease-1>", self.updateTopMutationsSpin);  # bind to trigger  
            self.TopMutationsScale.grid(row=2, column=3, sticky=W)
            self.updateTopMutationsScale()
                    
        elif (self.choice.get() == 1):  # destroy when not needed
            for element in self.optionElements :
                element.destroy() 
            self.optionElements = []
            
            if (self.frame2 != None):
                self.frame2.destroy()
    
    def updateAsaCutoffSpin(self, Event):
        """
        updates ASA slider for the display box variable
        """
        value = float(self.ASACutoffScale.get())
        self.ASACutoffText.set(value)
        
    def updateTopMutationsSpin(self, Event):
        """
        updates TopMutations slider for the display box variable
        """
        value = self.TopMutationsScale.get()
        self.TopMutationsText.set(value)
        
    def updateAsaCutoffScale(self):
        """
        updates input of display box for the display box variable
        """
        value = int(self.ASACutoff.get())
        
        self.ASACutoffScale.set(value)
    def updateTopMutationsScale(self):
        """
        updates input of display box for the display box variable
        """
        
        value = int(self.TopMutations.get())
        self.TopMutationsScale.set(value)
        
    def updateAsaFilterChanged(self, mode):
        """
        updates access to ASA slider/display box with the dropdown Menu
        """
        if(mode == "none"): 
            # set Spinbox and Slider disabled
            self.ASACutoff.configure(state=DISABLED)
            self.ASACutoffScale.configure(state=DISABLED)
        else:
            self.ASACutoff.configure(state=NORMAL)
            self.ASACutoffScale.configure(state=NORMAL)
        
    def fillInUI(self, parent):
        """
        Building GUI 
        You can choose a PDBID and their corresponding subid and chain
        """
        # creat window
        self.parent = parent
        self.frame1 = Frame(parent, bg="#d3ebf7")
        self.frame1.pack(padx=0, pady=0, expand=YES, fill=BOTH)
        
        # formations box
        self.labelframe1 = LabelFrame(self.frame1, text="Select Structure", bg="#d3ebf7")
        self.labelframe1.pack(expand="yes", padx=0, pady=5)
        
        # labels
        self.label1 = Label(self.labelframe1, text="PDB-ID:      ", bg="#d3ebf7").grid(row=0, column=0)
        self.label2 = Label(self.labelframe1, text="Substructure: ", bg="#d3ebf7").grid(row=1, column=0)
        self.label3 = Label(self.labelframe1, text="Chain:       ", bg="#d3ebf7").grid(row=2, column=0)
        
        self.loadStruct()
        
        self.PDB_IDs = self.structures.keys()  # list for the PDB_IDs dropdown
        if (len(self.PDB_IDs) > 0):
            self.Substructures = self.structures[self.PDB_IDs[0]]['submodels']  # list for the Substructures dropdown
            self.Chains = self.structures[self.PDB_IDs[0]]['chains'].keys()  # list for the Chains dropdown
   
        
        # PDB_IDs dropdown
        self.PDB_ID_Menu = ComboBox(self.labelframe1, listbox_width=30, dropdown=1, entry_width=25, scrolledlist_items=self.PDB_IDs, selectioncommand=self.PdbIdUpdateComboBox_)
        self.PDB_ID_Menu.grid(row=0, column=1)
        
        if (len(self.PDB_IDs) <= 0):
            self.Substructures = []
        # Substructures dropdown
        self.Substructure_Menu = ComboBox(self.labelframe1, listbox_width=30, dropdown=1, entry_width=25, scrolledlist_items=self.Substructures)
        self.Substructure_Menu.grid(row=1, column=1)
        
        self.Chains = ["*"]
        # Chains dropdown
        self.Chain_Menu = ComboBox(self.labelframe1, listbox_width=30, dropdown=1, entry_width=25, scrolledlist_items=self.Chains, selectioncommand=self.chainUpdateComboBox_)
        self.Chain_Menu.grid(row=2, column=1)
       

        self.label4 = Label(self.labelframe1, text="[* means all chains]", bg="#d3ebf7").grid(row=2, column=2)
        
        # formations box
        self.labelframe2 = LabelFrame(self.frame1, text="Select Task", bg="#d3ebf7")
        self.labelframe2.pack(expand="yes", padx=0, pady=5)
        
        # variable which know whith Radiobutton is selected
        self.choice = IntVar()
        self.choice.set(1)
        
        # Radiobutton
        self.rb1 = Radiobutton(self.labelframe2, text="Calculate a mutation sensitivity profile", variable=self.choice, value=1, command=self.fillOptionFrame, bg="#d3ebf7").pack(anchor=W)
        self.rb2 = Radiobutton(self.labelframe2, text="Evaluate specific mutations", variable=self.choice, value=2, command=self.fillOptionFrame, bg="#d3ebf7").pack(anchor=W)
        self.rb3 = Radiobutton(self.labelframe2, text="Scan for (de)stabilizing mutations", variable=self.choice, value=3, command=self.fillOptionFrame, bg="#d3ebf7").pack(anchor=W)
        # self.rb2.config(state = DISABLED)
        
        self.chimeraUpdateComboBox()
        if (len(self.PDB_IDs) > 0):
            self.PdbIdUpdateComboBox()
            if (len(self.PDB_IDs) > 0):
                self.PDB_ID_Menu.selectitem(str(self.PDB_IDs[0]))
            if (len(self.Substructures) > 0):
                self.Substructure_Menu.selectitem(self.Substructures[0])
            if (len(self.Chains) > 0):
                self.Chain_Menu.selectitem('*')
        
        
        
        

    
    
    def enableGUI(self, state=True):
        """
        When there is no pdb loaded and one klicks at sth in GUI it crashes.
        This function disables the components of the GUI to avoid it from crashing.
        If pdb is finally loaded it shall be enbaled again.
        This function shall be called when creating GUI and no pdb is loaded and when removing last loaded pdb from chimera.
        """
        
        if(state == False):
            self.PDB_ID_Menu.grid_forget()
            self.Substructure_Menu.grid_forget()
            self.Chain_Menu.grid_forget()
            self.buttonWidgets['Apply'].config(state=DISABLED)
        else:
            self.PDB_ID_Menu.grid(row=0, column=1)
            self.Substructure_Menu.grid(row=1, column=1)
            self.Chain_Menu.grid(row=2, column=1)
            self.buttonWidgets['Apply'].config(state=NORMAL)
        
            
    def getPdbListMaestro (self):
        """
        Returns a pair of the pdbid and the corresponding pdbstring of the .pdb-file
        """

        self.selID = self.PDB_ID_Menu.get()  # selected
        model = self.structures[self.selID]['ptrModel']  # durch ausgewaehltes model ersetzen
        
        #### added by Jonas
        
        # we need to display ID of current selected substructe in result
        curSelSub = self.Substructure_Menu.get()  # get current selected substructure (of drop down)
        self.selID = str(str(self.selID) + ";model=" + str(curSelSub))  # append substructure
        
        #### end added by Jonas
        
        mem = StringIO()
        chimera.pdbWrite([model], model.openState.xform, mem)
        mem.seek(0, 0)  # zurueck auf erste zeile (zeiger war auf der letzten zeile -> deshalb gabs keine ausgabe)
    
        self.pdbstr = mem.getvalue()
        mem.close()
        self.PDBpairs = []
        self.PDBpairs.append((self.selID, self.pdbstr))
        return self.PDBpairs        

chimera.dialogs.register(MaestroDialog.name, MaestroDialog)


#code duplication! 
def loadStructResult():
    """
    loads the structures from Chimera into the structures Object
    used when bioDApps loads a job 
    """

    curModels = chimera.openModels.list(modelTypes=[chimera.Molecule])  # current opened Molecule
        
    structures = {}
    for m in curModels:  # Loop to get all Molecule-names  
        if not structures.has_key(m.name):
            
            structures[m.name] = {}
            structures[m.name]['id'] = m.id
            structures[m.name]['submodels'] = []
            structures[m.name]['chains'] = {}
            structures[m.name]['chains']['*'] = [] 
            structures[m.name]['ptrModel'] = m  # Pointer to model to get the pdbstring easier
            # fill chains in map
            residues = m.residues
            for r in residues:
                if (r.id.chainId not in structures[m.name]['chains']):
                    structures[m.name]['chains'][r.id.chainId] = []
                # first create residue's name before inserting
                aaMap = AAMapping();
                aaLetter = aaMap.aaThreeToOne(str(r.type))
                residueName = str(aaLetter) + str(r.id.position) + "." + str(r.id.chainId)
                structures[m.name]['chains'][r.id.chainId].append(residueName)
                structures[m.name]['chains']['*'].append(residueName)
        # fill submodels in map
        structures[m.name]['submodels'].append(m.subid)
    return structures



def getResultById(jobId):
    """
    this method is called in the BioDapps Gui
    reload the structure Object with data from session
    getting the id for an job   
    opens the right ResultGUI for the task
    creates a result Gui
    @param jobid: Id of the job to get the right session/result
    """
    
    structures = loadStructResult();
    BioDAppsC = Connector.MaestroConnector()
    result = BioDAppsC.getResults(jobId)
    task = result['result']['task']
    
    # table = MaestroTable(structures)
    # table.setListData(result)

    if (task == 'scanMutations'):
        MSResult(structures, result)
    elif (task == 'evaluate'):
        MaestroTable(structures, result)
    elif(task == 'spots'):
        URL = "https://biwww.che.sbg.ac.at/biodapps/pl/rest" + result['result']['downloadPng'];
        MResultSensiProf(URL);





class MSSuperResult(ModelessDialog):
    '''
    Super class for result GUIs
    Depending on the job send by the Maestro GUI, the result is different, but they still have a lot in common
    is inherited from ModelessDialog
    '''
    
    buttons=('Close')
    
    def __init__(self,result = None, **kw):
        """
        sets all important variables also from subclasses 
        this superclass is used by the 3 ResultGUIS of Chimera
        _master.after is a element/function of the ModelessDialog, is used as invokeLater
        @param **kw: is needed for super() 
        @param result: dictionary which contains the result of the job
        """
        self.close = False
        self.result = result 
        self.progress = 0
        self.waiting = True
        ModelessDialog.__init__(self, **kw)
        self._master.after(100, self.render)

    def Close(self):   
        """
        To close the GUI 
        """       
        self.close = True
        ModelessDialog.Close(self)    
        self.destroy()
        
    def setResult(self, result):
        """
        setter for result 
        """
        self.result = result
        
    def fillGui(self):
        """
        is overrided by the used subclasses
        """
        pass
    
    def render(self):
        """
        is called every 1000ms after the ResultGUI is initiated
        if the ResultGUI is not closed or if there is no result 
        you see a progressbar otherwise the GUI will be closed 
        or filled with the given result
        """
        if self.close:
            ModelessDialog.Close(self)    
            self.destroy()
            return
        if self.result:
            for element in self.parent.winfo_children() :
                    element.destroy()
            self.fillGui()
        else:
            if not self.waiting:
                self.progressbar['value']= self.progress
            elif self.progress > 0:
                self.progressbar.stop()
                self.progressbar['mode']='determinate'
                self.progressbar['value']= self.progress
                self.waiting = False
            self._master.after(1000, self.render)
            
    def updateProgress(self, progress):
        """
        callback Mothed for BioDApps Connector 
        updates Progressbar of the ResultGUI
        """
        self.progress = progress

    def fillInUI(self, parent):
        """
        StandartGUI Interface of the Superclass (sets colors, Labels etc.)
        In the beginning it have only the Progressbar
        """
        self.parent = parent
        self.frame = Frame(self.parent, bg="#d3ebf7")
        self.frame.pack(padx=0, pady=0, expand=YES, fill=BOTH)
        self.labelframeProgress = LabelFrame(self.frame, text="Status", bg="#d3ebf7")
        self.labelframeProgress.pack(expand="yes", padx=0, pady=5)
        # bar:
        s = ttk.Style()
        s.theme_use('clam')
        s.configure("blue.Horizontal.TProgressbar", foreground='blue', background='blue')
        self.progressbar = ttk.Progressbar(self.labelframeProgress, style="blue.Horizontal.TProgressbar", length=300, mode='indeterminate')  # default range is 0 to 100
        self.progressbar.grid(row=0, column=1)
        self.progressbar.start()
        

class MResultSensiProf(MSSuperResult):
    """
    Generates and fills GUI for displaying results of the MaestroJob
    The Result is diplayed as a Picture of the Sensitive Profile
    
    inherited from ModelessDialog  
    """    
    
    title="MAESTRO Result - Sensitivity Profile"
    name="MAESTRO Result - Sensitivity Profile"
    
    
    
    def __init__(self,result = None, **kw):
        """
        Constructor
        Superclass init is called
        @param **kw: is needed for super() 
        @param result: dictionary which contains the result of the job
        """
        MSSuperResult.__init__(self, result)      
        
    def fillGui(self):
        """
        Fills GUI.
        GUI shows a Picture of Sensitive Profile
        """
        #creat window:
        self.frame1=Frame(self.parent, bg="#d3ebf7")
        self.frame1.pack(padx=0,pady=0,expand=1,fill=BOTH)
        
        #formations box:
        self.labelframe1 = LabelFrame(self.frame1, text="Sensitivity Profile", bg="#d3ebf7")
        self.labelframe1.pack(expand=1, padx=0, pady=5, fill=BOTH)
        URL = self.result
        #"http://chimera2016.projekte.fh-hagenberg.at/biodapps/rest/download/Maestro?src=jobs/297182df4cc4ed6ba4e71c32f42dad71/sensitivityProfile_297182df4cc4ed6ba4e71c32f42dad71.png"
        r = requests.get(URL)
        image = PIL.Image.open(StringIO(r.content))
        img = PIL.ImageTk.PhotoImage(image)
        #save ref to img so it doesn't get lost:
        labelSP = Label(image=img)
        labelSP.image = img # keep a reference!
        #labelSP.pack()    #may cause wrong dispalying of result
        
        #add png to textbox:
        #http://pmw.sourceforge.net/doc/ScrolledFrame.html:
        lw = Pmw.ScrolledFrame(self.labelframe1, labelpos='n',)    #ignore ERROR @UndefinedVariable
        lw.pack(expand=1, fill=BOTH)    #options expand and fill = To make the widget fill the entire parent, also if the user resizes the window http://effbot.org/tkinterbook/pack.htm

        cwSP = Button(lw.interior(),background="#d3ebf7", image=img)
        cwSP.pack(expand=1, fill=BOTH)
        
               
chimera.dialogs.register(MResultSensiProf.name, MResultSensiProf)


class MaestroTable(MSSuperResult):
    """
    Generates and fills GUI for displaying results of the MaestroJob
    The Result is diplayed as a table (residue,ddg,quality)
    It's possible to Highlight the residues 
    
    inherited from ModelessDialog  
    """
    
    title="Maestro Result"
    name="Maestro Result"
    def __init__(self, dictio, result = None, **kw):    
        """
        Constructor
        
        @param dictio: dictionary which contains all necessary data of the chosen molecule 
        (name,id,submodel,chain,residue,pointer) 
        @param **kw: is needed for super() 
        @param result: dictionary which contains the result of the job
        """    
        
        #fill in GUI:
        self.Res = {}
        self.dict = dictio
        MSSuperResult.__init__(self, result) 
        
        
        
    def setListData(self):        
        """
        fills the multilistbox of the ResultGUI with the data form the result
        """
        i = 0;       
        for entry in self.result['result']['substitutionlist']:   
            self.mlb.insert(tkc.END, (entry['substitution'],round(entry['ddG'],2) ,round(entry['quality'],2)))
            self.mlb.justify()
            i = i+1
                           
    def sel(self):
        """
        Defines what happens if the user select a residue in the List.
        Proofs if the selected residue is available and colors it 
        Its selected until lines of this residues are unselected
        Colors the selected residue.  
        """
        
        pdbname = self.result['result']['structure_info']
        prot = str(pdbname).split(",")[0]
        
        #### inserted by Jonas 28.10.16
        
        self.realsubid = str(prot).split(";model=")[1]
        prot = str(prot).split(";model=")[0]
        #### end inserted by Jonas 28.10.16
        
        id_ = self.dict[prot]['id']
        entry = self.mlb.getLastSelection()
        p = re.compile('^[A-Z]([0-9]+)\.([A-Z]*)') 
        resindex = p.match(entry).group(1)
        chains = p.match(entry).group(2)
        
        select = selection.OSLSelection("#" + str(id_))
        self.color = select.molecules()[0].color.rgba()
        
        if resindex not in self.Res:
            self.Res[resindex] = []
        self.Res[resindex].append(resindex)
        
        
        for chain in chains:
            
            sel = selection.OSLSelection("#" + str(id_) + "." + self.realsubid + ":" + resindex + "." + chain + " ") #### inserted by Jonas 28.10.16: changed self.subid to self.realsubid
            res = sel.residues()
            itemsSel = selection.ItemizedSelection()
            itemsSel.add(res[0])
            
            Midas.colordef('rgb_sel', (1, 0, 0, 1))
            Midas.color('rgb_sel', itemsSel)
            
        
    def deSel(self):
        """
        Defines what happens if the user deselects a residue in the List.
        Remove color the selected residue.
        """
        pdbname = self.result['result']['structure_info']
        protall = str(pdbname).split(",")[0]
        prot = str(protall).split(";")[0]
        self.subid = str(protall).split(";model=")[1]
        id_ = self.dict[prot]['id']
        
        
        entry = self.mlb.getLastDeSelection()
        p = re.compile('^[A-Z]([0-9]+)\.([A-Z]*)') 
        resindex = p.match(entry).group(1)
        chains = p.match(entry).group(2)
        #command = ("color \"white\" ")
        
        resList = self.Res[resindex]
        del resList[-1]
        
        if len(resList) == 0:
            #from chimera import selection
            for chain in chains:
                sele = selection.OSLSelection("#" + str(id_) + "." + self.subid + ":" + resindex + "." + chain + " ")
                res = sele.residues()
                
                itemsSel = selection.ItemizedSelection()
                itemsSel.add(res[0])
                Midas.colordef('rgb_sele', (self.color))
                Midas.color('rgb_sele', itemsSel)
           
       
    def fillGui(self):
        """
        Fills GUI wtih components.
        GUI has a Table(MultiListBox) (residue,ddg,quality)
        """
        self.frame1=Frame(self.parent, bg="#d3ebf7")
        self.frame1.pack(padx=0,pady=0,expand=YES,fill=BOTH)
        
        #formations box
        self.labelframe1 = LabelFrame(self.frame1, text="Summarize", bg="#d3ebf7")
        self.labelframe1.pack(expand="yes", padx=0, pady=5)
               
        self.mlb = mlb.MultiListbox(self.labelframe1, True,  (('substituition', 20, StringType, tkc.LEFT), #(name,breite,Typ,
                                                              ('     ddg     ', 3, StringType, tkc.CENTER),
                                                              ('   quality   ', 3, StringType, tkc.CENTER)),self.sel,self.deSel)
     
        self.mlb.pack(side=RIGHT, fill=Y)
        #self.mlb.insert(tkc.END, ("S2.VW{A},F7.VW{A},F12.VW{H}",2.9380408347574885 ,0.7293644063075038))
        self.mlb.justify()
        self.setListData()
                     
chimera.dialogs.register(MaestroTable.name, MaestroTable) 


class MSResult(MSSuperResult):
   
    title="Mutationsscan Result"
    name="Mutationsscan"
    
    def __init__(self, dictStruct, result = None, **kw):   
        """
        Constructor
        
        @param dictStruct: dictionary which contains all necessary data of the chosen molecule 
        (name,id,submodel,chain,residue,pointer) 
        @param result: result to build the gui can also be given in the constructor
        """    
        MSSuperResult.__init__(self, result)
        self.task = None
        if self.result:
            self.task = result['result']['task']
        self.dict = dictStruct
        self.Res = {} 
       
    
    def setResult(self, result):
        """ 
        setter of result and task 
        """
        self.result = result
        self.task = result['result']['task']
    
    def setListData(self):   
        """
        go through every entry of the result and fills the multilistbox the data form the result
        """
        for entry in self.result['result']['topMutants']:  
            if (self.task == 'scanSSBonds'):
                self.mlb.insert(tkc.END, (entry[0],round(entry[1],2) ,round(entry[2],2) ,round(entry[3])))
            else:
                self.mlb.insert(tkc.END, (entry[0],round(entry[1],2) ,round(entry[2],2)))
            self.mlb.justify()
            
    def sel(self):
        """
        Defines what happens if the user select a residue in the List.
        Proofs if the selected residue is available and colors it 
        Its selected until lines of this residues are unselected
        Colors the selected residue.      
        """
        pdbinfo = self.result['result']['structure_info'] 
        prot = str(pdbinfo).split(';')[0]        
        partial = str(pdbinfo).split(';')[1]
        model = str(partial).split(',')[0] 
        
        subid = str(model).split('=')[1]
        OSLid = self.dict[prot]['id']
        
        select = selection.OSLSelection("#" + str(OSLid))
        self.color = select.molecules()[0].color.rgba()    
      
        entry = self.mlb.getLastSelection()
        p = re.findall('(\d+)\.(\w+)',entry) 
        
        ph = re.findall('\d+',entry)
        
        for res in ph:
            if res not in self.Res:
                self.Res[res] = []
            self.Res[res].append(0)
        
        for pair in p:
            resindex = pair[0]
            chains = pair[1]
                       
            for chain in chains:       
                sel = selection.OSLSelection("#" + str(OSLid) + "." + subid + ":" + resindex + "." + chain + " ")
                res = sel.residues()
                itemsSel = selection.ItemizedSelection()
                itemsSel.add(res[0])
                Midas.colordef('rgb_sel', (1, 0, 0, 1))
                Midas.color('rgb_sel', itemsSel)
        
    def deSel(self):
        """
        Defines what happens if the user deselect a residue in the List.
        Remove color the selected residue.
        """
        
        pdbinfo = self.result['result']['structure_info'] 
        prot = str(pdbinfo).split(';')[0]        
        partial = str(pdbinfo).split(';')[1]
        model = str(partial).split(',')[0] 
        
        subid = str(model).split('=')[1]
        OSLid = self.dict[prot]['id']
        
        select = selection.OSLSelection("#" + str(OSLid))
        self.color = select.molecules()[0].color.rgba()
      
        entry = self.mlb.getLastDeSelection()
        p = re.findall('(\d+)\.(\w+)',entry) 
        
        
        #ph = re.findall('\d+',entry)
        resList = []

        for pair in p:
            resindex = pair[0]
            chains = pair[1]
            resList = self.Res[resindex]
            del resList[-1]
            if len(resList) == 0: 
                for chain in chains:       
                    sel = selection.OSLSelection("#" + str(OSLid) + "." + subid + ":" + resindex + "." + chain + " ")
                    res = sel.residues()
                    itemsSel = selection.ItemizedSelection()
                    itemsSel.add(res[0])
                    Midas.colordef('rgb_sele', (self.color))
                    Midas.color('rgb_sele', itemsSel)
        
        
            
                
             
    def fillGui(self):  
        """
        Fills GUI.
        GUI has a Table(MultiListBox) (residue,ddg,quality)
        """    
         
        self.frame1=Frame(self.parent, bg="#d3ebf7")
        self.frame1.pack(padx=0,pady=0,expand=YES,fill=BOTH)
        
        self.labelframe1 = LabelFrame(self.frame1, text="Summarize", bg="#d3ebf7")
        self.labelframe1.pack(expand="yes", padx=0, pady=5)
        
        if (self.task != 'scanSSBonds'):    
            self.mlb = mlb.MultiListbox(self.labelframe1, True,  (('substituition', 30, StringType, tkc.LEFT),
                                                                  ('     ddg     ', 3, StringType, tkc.CENTER),
                                                                  ('   quality   ', 3, StringType, tkc.CENTER)),self.sel,self.deSel)
        else:
            self.mlb = mlb.MultiListbox(self.labelframe1, True,  (('substituition', 30, StringType, tkc.LEFT),
                                                                  ('     ddg     ', 3, StringType, tkc.CENTER),
                                                                  ('   quality   ', 3, StringType, tkc.CENTER),
                                                                  ('  Bond Score ', 3, StringType, tkc.CENTER)),self.sel,self.deSel)   
        self.mlb.pack(side=RIGHT, fill=Y)
        self.mlb.justify()
        self.setListData()
                     
chimera.dialogs.register(MSResult.name, MSResult) 