#Josef Laimer, University of Salzburg, 2010
import chimera
from chimera.baseDialog import ModelessDialog
from chimera import dialogs
from chimera.resCode import *
 
import Tkinter
from Tkinter import *
import Pmw
from Pmw import ComboBox
from Pmw import NoteBook
from Pmw import ScrolledFrame
import chimera.extension
from chimera.extension import ScrolledGrid
from time import *
 
#import AccSurfSelection.gui as asagui
import BioDApps_AccSurfSelectionWeb.gui as asagui
 
DEFAULT_PLOT_WIDTH = 800
DEFAULT_PLOT_HEIGHT = 250
DEFAULT_SUMPLOT_HEIGHT = 50
 
DEFAULT_MODE = 1
MUTANT_MODE = 2
 
DESCR_SPACE = 130
 
LETTER_WIDTH=11
LETTER_HEIGHT=20
FONT=('courier', 14)
 
FILTER_COLOR='darkgreen'
 
class MaskBuilderDialog (ModelessDialog):
    name = 'Mask Builder'  
    title = 'Mask Builder'
    buttons = ('Close')  
     
    def __init__(self, mode=DEFAULT_MODE, molOsl=None, caller=None):
        self.mode = mode
        self.selOSL = molOsl
        self.caller = caller
       
        self.models = {}
        self.modelNames = {}
        self.molecules = chimera.openModels.list(modelTypes=[chimera.Molecule])
       
        self.selModel = None
        self.selSubModel = None
        self.showSubModel = False
       
        self.operatorVar = StringVar()
        self.operatorVar.set('AND')
        self.operator = 'OR'
       
        self.filterLine = 0
        self.filterLineObjDel = {}
        self.filterLineObjInv = {}
       
        #canvas objects
        self.filterObj = {}
    
        self.filter = {}
       
        self.sumFilterObj = []
        self.filterResidues = []
       
        #filter frame methods
        self.filterTypes = {
                            '': None,
                            'Amino Acid': self.__drawAATypeFilter,
                            'ASA': self.__drawASAFilter,
                            'Sec. Structure': self.__drawSSFilter,
                            'Chimera Selection':self.__drawSelectionFilter,  
                            'User': self.__drawUserFilter                        
                            }
       
       
        ModelessDialog.__init__(self, resizable=False)    
       
        self._closeModelHandlerID = chimera.openModels.addRemoveHandler(self._changeModelHandler, None)
        self._addModelHandlerID = chimera.openModels.addAddHandler(self._changeModelHandler, None)
 
        self.__fillModels()
        self.__fillModelsCB()  
       
        if len(self.modelNames.keys()) > 0:
            if self.mode == MUTANT_MODE:
                if not self.filter.has_key(self.selOSL):
                    self.filter[self.selOSL] = {}
                self.__drawMolecule()
   
            self.__selectOperator()
       
   
    def Close(self):
        if self._closeModelHandlerID != None:
            chimera.openModels.deleteRemoveHandler(self._closeModelHandlerID)
            self._closeModelHandlerID = None
        if self._addModelHandlerID != None:
            chimera.openModels.deleteAddHandler(self._addModelHandlerID)
            self._addModelHandlerID = None
        self.destroy()  
 
    #**********************************************************************************
    #eventhandler
    #**********************************************************************************
 
   
    #outside trigger
    #===============
    def _changeModelHandler(self,trigger, additional, molList):
        self.molecules = chimera.openModels.list(modelTypes=[chimera.Molecule])
        self.__fillModels()
        self.__fillModelsCB()
 
           
    #inside trigger
    #==============
   
    def __selectModel(self, entry):
        if entry == None:
            return
       
        if entry != self.selModel:
            self.selModel = entry
            self.selOSL = "#" + str(self.modelNames[self.selModel])
            submodels = self.models[self.modelNames[self.selModel]]['submodels'].keys()
            if len(submodels) > 1:
                submodels.sort()
                if not self.showSubModel:
                    self.lSubModel = Label(self.fOptions, text='Submodel')
                    self.cbSubModels = ComboBox(self.fOptions, listbox_width=10, \
                                                dropdown=1, entry_width=10, \
                                                selectioncommand=self.__selectSubmodel, \
                                                scrolledlist_items=[])
                    self.lSubModel.grid(row=0, column=2, sticky=W, padx=5, pady=5)
                    self.cbSubModels.grid(row=0, column=3, sticky=W, padx=5, pady=5)
               
                self.showSubModel = True
                self.cbSubModels.component('scrolledlist').setlist(submodels)
                self.cbSubModels.selectitem(0)
               
                self.selOSL += "." +  str(submodels[0])
            else:
                if self.showSubModel:
                    self.lSubModel.destroy()
                    self.cbSubModels.destroy()
                   
                self.showSubModel = False
           
            self.selSubModel = submodels[0]
           
            if not self.filter.has_key(self.selOSL):
                self.filter[self.selOSL] = {}
            self.__drawMolecule()
            #self.__fillResult() TODO
            self.__redrawSumLine()
            self.__redrawFilters()
 
    def __selectSubmodel(self, entry):
        self.selSubModel = entry
        self.selOSL = "#" + str(self.modelNames[self.selModel]) + "." + str(entry)
        if not self.filter.has_key(self.selOSL):
            self.filter[self.selOSL] = {}
        self.__drawMolecule()
        #self.__fillResult() TODO
        self.__redrawSumLine()
        self.__redrawFilters()      
 
    def __selectOperator(self, event=None):
        if not self.operatorVar.get() == self.operator:
            self.operator = self.operatorVar.get()            
            self.__redrawSumLine()
           
           
    def __canvasMotion(self, event):
        obj = event.widget.find_withtag(CURRENT)
       
        self.cPlot.delete(self.canvasLegend)
       
        if len(obj) < 1:# and not self.canvasLegend == None:
            return
 
        if self.filterObj.has_key(obj[0]):
            self.canvasLegend = self.cPlot.create_text(self.cPlot.canvasx(event.x), \
                                                       self.cPlot.canvasy(event.y), \
                                                       text=self.filterObj[obj[0]], \
                                                       anchor=S, fill='blue')
 
    def __canvasClick(self, event):
        obj = event.widget.find_withtag(CURRENT)
       
        if len(obj) < 1:
            return
 
        obj = obj[0]
       
        if self.filterLineObjDel.has_key(obj):
            name = self.filterLineObjDel[obj]['name']
            line = self.filterLineObjDel[obj]['line']
            for o in self.filterLineObjDel[obj]['obj']:
                self.cPlot.delete(o)
               
            self.filter[self.selOSL].pop(name)
            self.filterLineObjDel.pop(obj)
           
            for k in self.filterLineObjDel.keys():
                if self.filterLineObjDel[k]['line'] > line:
                    for o in self.filterLineObjDel[k]['obj']:
                        self.cPlot.move(o, 0, LETTER_HEIGHT*-1)                    
                    self.filterLineObjDel[k]['line'] = self.filterLineObjDel[k]['line'] - 1
                   
            for obj in self.sumObjs:
                self.cPlot.move(obj, 0, LETTER_HEIGHT*-1)
       
            for obj in self.filterLineObjInv.keys():
                if self.filterLineObjInv[obj]['line'] > line:
                    self.filterLineObjInv[obj]['line'] = self.filterLineObjInv[obj]['line'] -1  
         
            self.filterLine -= 1
        elif self.filterLineObjInv.has_key(obj):
            name = self.filterLineObjInv[obj]['name']
            residues = self.filterLineObjInv[obj]['residues']
            remobj = self.filterLineObjInv[obj]['remobj']
            y = self.filterLineObjInv[obj]['line'] * LETTER_HEIGHT
            for o in  self.filterLineObjInv[obj]['obj']:
                try:
                    self.cPlot.delete(o)
                    self.filterLineObjDel[remobj]['obj'].remove(o)                    
                except:
                    pass
               
            filter = []
            mol = chimera.selection.OSLSelection(self.selOSL)
            mol = mol.molecules()[0]
            seqs = mol.sequences()
           
            i = 0
   
            self.filterLineObjInv[obj]['obj'] = []            
            self.filterLineObjInv[obj]['residues'] = []
   
            for s in seqs:
                i += 1
                x = DESCR_SPACE + i * LETTER_WIDTH
               
                for r in s.residues:
                    if r == None:
                        continue
                    i += 1
                    l1 = protein3to1[r.type]
                    x = DESCR_SPACE + i * LETTER_WIDTH
                   
                    if r not in residues:
                        filter.append(True)
                        x1 = x - LETTER_WIDTH/2
                        x2 = x + LETTER_WIDTH/2
                        y1 = y - LETTER_HEIGHT/2
                        y2 = y + LETTER_HEIGHT/2
                        rect = self.cPlot.create_rectangle(x1, y1, x2, y2, outline=FILTER_COLOR, width=2, fill=None)
                        self.filterLineObjDel[remobj]['obj'].append(rect)
                        self.filterLineObjInv[obj]['obj'].append(rect)
                        self.filterLineObjInv[obj]['residues'].append(r)
                    else:
                        filter.append(False)
                       
            self.filter[self.selOSL][name] = filter
        self.__redrawSumLine()
 
    def __selectFilter(self, entry):
        if not entry == self.lastFilterMethod:
            for e in self.filterElements:
                e.destroy()
       
            if not entry == '':
                self.filterTypes[entry]()
               
   
    def __selectResidues(self, event=None):
        itemSel = chimera.selection.ItemizedSelection()
        itemSel.add(self.filterResidues)
        chimera.tkgui.selectionOperation(itemSel)    
 
           
    def __returnMask(self, event=None):
        self.caller.addMask(self.filterResidues, self.selOSL)
 
    #**********************************************************************************
    #helper
    #**********************************************************************************
   
    def __fillModels (self):
        self.models = {}
        self.modelNames = {}
        for m in self.molecules:
            if not self.models.has_key(m.id):
                self.models[m.id] = {}
                self.models[m.id]['submodels'] = {}
                self.models[m.id]['name'] = m.name
                self.models[m.id]['longName'] = '#' + str(m.id) + ' - ' + m.name
                self.modelNames[self.models[m.id]['longName']] = m.id
               
            self.models[m.id]['submodels'][m.subid] = m          
           
 
    def __fillModelsCB (self):
        if not self.mode == DEFAULT_MODE:
            return
        sortModelNames = []
        modelIds = self.models.keys()
        modelIds.sort()
        for k in modelIds:
            sortModelNames.append(self.models[k]['longName'])
 
        self.cbModels.component('scrolledlist').setlist(sortModelNames)
       
        if sortModelNames.count(self.selModel) > 0:
            self.cbModels.selectitem(self.selModel)
        elif len(sortModelNames) > 0:
            self.cbModels.selectitem(0)
            self.__selectModel(sortModelNames[0])
 
   
    def __redrawSumLine (self):
        for obj in self.sumFilterObj:
            try:
                self.sumObjs.remove(obj)
                self.cPlot.delete(obj)
            except:
                pass
           
        self.sumFilterObj = []
        self.filterResidues = []
       
        sumline = self.filterLine + 2
       
        y = sumline * LETTER_HEIGHT
 
 
        mol = chimera.selection.OSLSelection(self.selOSL)
        mol = mol.molecules()[0]
        seqs = mol.sequences()
 
        i = 0
        filterI = -1
 
       
        for s in seqs:
            i += 1
            x = DESCR_SPACE + i * LETTER_WIDTH
                                             
            for r in s.residues:
                if r == None:
                    continue
               
                i += 1
                filterI += 1
                l1 = protein3to1[r.type]
                x = DESCR_SPACE + i * LETTER_WIDTH
               
                if self.__passFilter(filterI):
                    x1 = x - LETTER_WIDTH/2
                    x2 = x + LETTER_WIDTH/2
                    y1 = y - LETTER_HEIGHT/2
                    y2 = y + LETTER_HEIGHT/2
                    rect = self.cPlot.create_rectangle(x1, y1, x2, y2, outline=FILTER_COLOR, width=2, fill=None)
               
                    self.filterObj[rect] = r.oslIdent()
                    self.sumObjs.append(rect)
                    self.sumFilterObj.append(rect)
                   
                    self.filterResidues.append(r)
   
    def __passFilter (self, column):
        colFilter = []
        for k in self.filter[self.selOSL].keys():
            colFilter.append(self.filter[self.selOSL][k][column])
           
        if self.operator == 'AND':
            return colFilter.count(False) < 1
        else:
            return colFilter.count(True) > 0
 
   
    def __drawLine (self, line):
        if self.filterLine < 1:
            self.__drawSSline()
           
    def __drawSSline(self):
 
        if self.selOSL == None or self.selOSL == '':
            return
       
        mol = chimera.selection.OSLSelection(self.selOSL)
        mol = mol.molecules()[0]
        seqs = mol.sequences()
        self.filterLine += 1
        i = 0
 
        ssColors = {
                    'turn': 'darkgray',
                    'helix': 'red',
                    'strand': 'blue'
                    }
 
        y = self.filterLine * LETTER_HEIGHT
        for s in seqs:
            i += 1
            x = DESCR_SPACE + i * LETTER_WIDTH
           
            chainLetter = self.cPlot.create_text(x, y, text=s.chain, font=FONT)            
            self.filterObj[chainLetter] = 'start of chain ' + s.chain
           
            lastSStype = ''
            startSS = i
                       
            for r in s.residues:
                if r == None:
                    continue
               
                type = ''
                if r.isHelix:
                    type='helix'
                elif r.isStrand:
                    type='strand'
                else:
                    type='turn'
               
                i += 1
                if lastSStype != type:
                    if lastSStype != '':
                        xStart = DESCR_SPACE + startSS * LETTER_WIDTH - LETTER_WIDTH/2
                        xEnd = DESCR_SPACE + (i-1) * LETTER_WIDTH + LETTER_WIDTH/2
                        line = self.cPlot.create_line(xStart, y, xEnd, y, fill=ssColors[lastSStype], width=4)
                        self.filterObj[line] = lastSStype
                    lastSStype = type
                    startSS = i
           
            if lastSStype != '':
                xStart = DESCR_SPACE + startSS * LETTER_WIDTH - LETTER_WIDTH/2
                xEnd = DESCR_SPACE + (i) * LETTER_WIDTH + LETTER_WIDTH/2
                line = self.cPlot.create_line(xStart, y, xEnd, y, fill=ssColors[lastSStype], width=4)
                self.filterObj[line] = lastSStype
               
        self.cPlot.config(scrollregion=(0, 0, (i+1) * LETTER_WIDTH + DESCR_SPACE, self.filterLine * LETTER_HEIGHT))
        self.columns = i
       
    def __drawSumLine(self):
        sumline = self.filterLine + 1
        y = sumline * LETTER_HEIGHT
        xEnd = DESCR_SPACE + self.columns * LETTER_WIDTH
        self.sumObjs.append(self.cPlot.create_line(LETTER_WIDTH/2, y, xEnd+LETTER_WIDTH/2, y))
       
        sumline += 1
        y = sumline * LETTER_HEIGHT
 
 
        mol = chimera.selection.OSLSelection(self.selOSL)
        mol = mol.molecules()[0]
        seqs = mol.sequences()
 
        i = 0
       
 
        for s in seqs:
            i += 1
            x = DESCR_SPACE + i * LETTER_WIDTH
           
            chainLetter = self.cPlot.create_text(x, y, text='|', font=FONT)            
            self.filterObj[chainLetter] = 'start of chain ' + s.chain
            self.sumObjs.append(chainLetter)
                       
            for r in s.residues:
                if r == None:
                    continue
                i += 1
                l1 = protein3to1[r.type]
                x = DESCR_SPACE + i * LETTER_WIDTH
                resLetter = self.cPlot.create_text(x, y, text=l1, font=FONT)
                self.filterObj[resLetter] = r.oslIdent()
                self.sumObjs.append(resLetter)
   
   
    def __addFilterLine (self, name, residues):
        #move SumLine
        self.__moveSumLine()
       
        #draw name
        x = 10
        self.filterLine += 1
        y = self.filterLine * LETTER_HEIGHT
       
        remObj = self.cPlot.create_rectangle(x-LETTER_WIDTH/2, y-LETTER_HEIGHT/2, x+LETTER_WIDTH/2, y+LETTER_WIDTH/2, fill='red')
        x += LETTER_WIDTH + 2
        invObj = self.cPlot.create_rectangle(x-LETTER_WIDTH/2, y-LETTER_HEIGHT/2, x+LETTER_WIDTH/2, y+LETTER_WIDTH/2, fill='blue')
       
#        self.filterObj[remObj] = 'remove filter'
#        self.filterObj[invObj] = 'invert filter'
        self.filterLineObjDel[remObj] = {'line': self.filterLine, 'obj':[], 'name': name}
        self.filterLineObjDel[remObj]['obj'].append(invObj)
        self.filterLineObjDel[remObj]['obj'].append(remObj)
        self.filterLineObjInv[invObj] = {'line': self.filterLine, 'obj':[], 'residues':[], 'name': name, 'remobj': remObj}
       
        x += LETTER_WIDTH + 2
       
        shortName = name
        if len(shortName) > 15:
            shortName = shortName[0:15]
                   
        descObj = self.cPlot.create_text(x, y, text=shortName, anchor=W)
        self.filterObj[descObj] = name
        self.filterLineObjDel[remObj]['obj'].append(descObj)
       
        #draw sequence
        filter = []
        mol = chimera.selection.OSLSelection(self.selOSL)
        mol = mol.molecules()[0]
        seqs = mol.sequences()
       
        i = 0
 
        for s in seqs:
            i += 1
            x = DESCR_SPACE + i * LETTER_WIDTH
           
            chainLetter = self.cPlot.create_text(x, y, text='|', font=FONT)            
            self.filterObj[chainLetter] = 'start of chain ' + s.chain
            self.filterLineObjDel[remObj]['obj'].append(chainLetter)
                       
            for r in s.residues:
                if r == None:
                    continue
                i += 1
                l1 = protein3to1[r.type]
                x = DESCR_SPACE + i * LETTER_WIDTH
               
                if residues.count(r) > 0:
                    filter.append(True)
                    x1 = x - LETTER_WIDTH/2
                    x2 = x + LETTER_WIDTH/2
                    y1 = y - LETTER_HEIGHT/2
                    y2 = y + LETTER_HEIGHT/2
                    rect = self.cPlot.create_rectangle(x1, y1, x2, y2, outline=FILTER_COLOR, width=2, fill=None)
                    self.filterLineObjDel[remObj]['obj'].append(rect)
                    self.filterLineObjInv[invObj]['obj'].append(rect)
                    self.filterLineObjInv[invObj]['residues'].append(r)
                else:
                    filter.append(False)
 
                resLetter = self.cPlot.create_text(x, y, text=l1, font=FONT)
                self.filterObj[resLetter] = r.oslIdent()
                self.filterLineObjDel[remObj]['obj'].append(resLetter)
                   
        self.filter[self.selOSL][name] = filter
        #redraw sum  
        self.__redrawSumLine()
   
   
    def __redrawFilters (self):
        filters = self.filter[self.selOSL]
       
        for name in filters.keys():
            #move SumLine
            self.__moveSumLine()
 
            #draw name
            x = 10
            self.filterLine += 1
            y = self.filterLine * LETTER_HEIGHT
       
            remObj = self.cPlot.create_rectangle(x-LETTER_WIDTH/2, y-LETTER_HEIGHT/2, x+LETTER_WIDTH/2, y+LETTER_WIDTH/2, fill='red')
            x += LETTER_WIDTH + 2
            invObj = self.cPlot.create_rectangle(x-LETTER_WIDTH/2, y-LETTER_HEIGHT/2, x+LETTER_WIDTH/2, y+LETTER_WIDTH/2, fill='blue')
       
            self.filterLineObjDel[remObj] = {'line': self.filterLine, 'obj':[], 'name': name}
            self.filterLineObjDel[remObj]['obj'].append(invObj)
            self.filterLineObjDel[remObj]['obj'].append(remObj)
            self.filterLineObjInv[invObj] = {'line': self.filterLine, 'obj':[], 'residues':[], 'name': name, 'remobj': remObj}
       
            x += LETTER_WIDTH + 2
       
            shortName = name
            if len(shortName) > 15:
                shortName = shortName[0:15]
                   
            descObj = self.cPlot.create_text(x, y, text=shortName, anchor=W)
            self.filterObj[descObj] = name
            self.filterLineObjDel[remObj]['obj'].append(descObj)
           
            #draw sequence
            filter = filters[name]
            mol = chimera.selection.OSLSelection(self.selOSL)
            mol = mol.molecules()[0]
            seqs = mol.sequences()
       
            i = 0
            fiIndex = -1
   
            for s in seqs:
                i += 1
                x = DESCR_SPACE + i * LETTER_WIDTH
               
                chainLetter = self.cPlot.create_text(x, y, text='|', font=FONT)            
                self.filterObj[chainLetter] = 'start of chain ' + s.chain
                self.filterLineObjDel[remObj]['obj'].append(chainLetter)
               
                           
                for r in s.residues:
                    if r == None:
                        continue
                    i += 1
                    l1 = protein3to1[r.type]
                    x = DESCR_SPACE + i * LETTER_WIDTH
                   
                    fiIndex += 1
                    if filter[fiIndex]:
                        x1 = x - LETTER_WIDTH/2
                        x2 = x + LETTER_WIDTH/2
                        y1 = y - LETTER_HEIGHT/2
                        y2 = y + LETTER_HEIGHT/2
                        rect = self.cPlot.create_rectangle(x1, y1, x2, y2, outline=FILTER_COLOR, width=2, fill=None)
                        self.filterLineObjDel[remObj]['obj'].append(rect)
                        self.filterLineObjInv[invObj]['obj'].append(rect)
                        self.filterLineObjInv[invObj]['residues'].append(r)                        
   
                    resLetter = self.cPlot.create_text(x, y, text=l1, font=FONT)
                    self.filterObj[resLetter] = r.oslIdent()
                    self.filterLineObjDel[remObj]['obj'].append(resLetter)
           
   
   
    def __moveSumLine(self):
        for obj in self.sumObjs:
            self.cPlot.move(obj, 0, LETTER_HEIGHT)
   
    def __drawMolecule(self):
        self.cPlot.delete(ALL)
        self.filterLineObjDel = {}
        self.filterLineObjInv = {}
        self.filterLine = 0
       
        #canvas objects
        self.filterObj = {}
       
        self.columns = 0
        self.sumObjs = []
        self.__drawSSline()
        self.__drawSumLine()
 
    #**********************************************************************************
    #draw filter frames
    #**********************************************************************************
    def __drawAATypeFilter(self):
        fAAFilter = Frame(self.fFilterDetail, bg="#d3ebf7")
        fAAFilter.pack(side=TOP, fill=BOTH, expand=NO)
        self.filterElements.append(fAAFilter)
       
        strName = StringVar()
        strName.set('AA type')
        lName = Label(fAAFilter, text='Name ')
        eName = Entry(fAAFilter, textvariable=strName, width=35, state=NORMAL)
       
        lName.grid(row=0, column=0, sticky=W, padx=5, pady=5)
        eName.grid(row=0, column=1, sticky=W, padx=0, pady=5, columnspan=5)
       
        lAA = Label(fAAFilter, text='Amino acids ')
 
        sbAA = Scrollbar(fAAFilter)
        lbAA = Listbox(fAAFilter, height=5, width=15, yscrollcommand=sbAA.set)
        sbAA.config(command=lbAA.yview)
        aa3l = protein3to1.keys()
        aa3l.sort()
        

        
        for aa in aa3l:
            lbAA.insert(END, aa)
        
        # BEGIN **********************************************************************************
        # edited by Jonas on 29.11.2016 (first)
        
        lbAA.insert(END,"HYDROPHOBIC")
        lbAA.insert(END,"POLAR")
        lbAA.insert(END,"AROMATIC")
        lbAA.insert(END,"ALIPHATIC")
        lbAA.insert(END,"CHARGED")
        lbAA.insert(END,"POSITIVE")
        lbAA.insert(END,"NEGATIVE")
        lbAA.insert(END,"SULPHURCONT")
        lbAA.insert(END,"TINY")
        lbAA.insert(END,"SMALL")
        lbAA.insert(END,"LARGE")
        
        # END **********************************************************************************           
       
        
        sbAA2 = Scrollbar(fAAFilter)
        lbAA2 = Listbox(fAAFilter, height=5, width=15, yscrollcommand=sbAA2.set)
        sbAA2.config(command=lbAA2.yview)
        
       
        lAA.grid(row=2, column=0, sticky=W+N, padx=5, pady=5)
        lbAA.grid(row=2, column=1, sticky=W, padx=0, pady=5)
        sbAA.grid(row=2, column=2, sticky=N+S, padx=0, pady=5)
       
        lbAA2.grid(row=2, column=4, sticky=W, padx=0, pady=5)
        sbAA2.grid(row=2, column=5, sticky=N+S, padx=0, pady=5) 
        
        # BEGIN **********************************************************************************
        # edited by Jonas on 06.12.2016 (first)
        
        self.oldTime = -1
        #lbAA2.configure(state=DISABLED)
        
        # END ********************************************************************************** 
         
        def addAA(event=None):          
            selIndex = lbAA.curselection()
            if selIndex > -1:
                aa = lbAA.get(selIndex)
                setList(aa);
                    
              
                

                
                
       
        def remAA(event=None):
            
            #newTime = time()
            #if ((self.oldTime+0.5 < newTime) or (self.oldTime == -1) ): 
                #lbAA2.configure(state=NORMAL)       
                selIndex =lbAA2.curselection()
                #selIndex = lbAA.get(ACTIVE)
                print (type(selIndex)) # -> tuple
                print (selIndex)
                #print (type(selIndex[0])) # -> tuple
                if ((len(selIndex) != 0)):
                    selIndex = selIndex[0]
                    aa = lbAA2.get(selIndex)
                    lbAA2.delete(selIndex)
                    #self.oldTime = newTime;  
                    #lbAA2.configure(state=DISABLED)
           
        def clearList(event=None):
            lbAA2.delete(0, END)
           
        def addFilter(event=None):
            filterRes =[]
            mol = chimera.selection.OSLSelection(self.selOSL)
            mol = mol.molecules()[0]
            res = mol.residues
           
            aa = lbAA2.get(0, END)

            aaList = []
            for a in aa:
                aaList.append(a)
            
            for r in res:
                if aaList.count(r.type) > 0:
                    filterRes.append(r)
                  
            name = strName.get()
            if self.filter[self.selOSL].has_key(name):
                i = 2
                while self.filter[self.selOSL].has_key(name + " " + str(i)):
                    i += 1
                   
                name = name + " " + str(i)
            
            self.__addFilterLine(name, filterRes)
 
        lbAA.bind('<Double-Button-1>', addAA)
        lbAA2.bind('<Double-Button-1>', remAA)
       
        btnClearList = Button(fAAFilter, text='Clear list', command=clearList, state=NORMAL)
        btnAddFilter = Button(fAAFilter, text='Add filter', command=addFilter, state=NORMAL)
        btnAddFilter.grid(row=10, column=10, sticky=W, padx=10, pady=10)
        btnClearList.grid(row=10, column=1, sticky=W, padx=10, pady=10)
        
        
        def setList(aa):
            if (aa == "HYDROPHOBIC"):
                lbAA2.insert(END, "ALA");
                lbAA2.insert(END, "CYS");
                lbAA2.insert(END, "GLY");
                lbAA2.insert(END, "ILE");
                lbAA2.insert(END, "LEU");
                lbAA2.insert(END, "MET");
                lbAA2.insert(END, "PHE");
                lbAA2.insert(END, "PRO");
                lbAA2.insert(END, "TRP");
                lbAA2.insert(END, "VAL");
                
            if (aa == "POLAR"):
                lbAA2.insert(END, "ASP");
                lbAA2.insert(END, "GLU");
                lbAA2.insert(END, "ARG");
                lbAA2.insert(END, "LYS");
                lbAA2.insert(END, "HIS");
                lbAA2.insert(END, "ASN");
                lbAA2.insert(END, "GLN");
                lbAA2.insert(END, "SER");
                lbAA2.insert(END, "THR");
                lbAA2.insert(END, "TYR");
            
            elif (aa == "AROMATIC"):
                lbAA2.insert(END, "HIS");
                lbAA2.insert(END, "PHE");
                lbAA2.insert(END, "TRP");
                lbAA2.insert(END, "TYR");
                
            elif (aa == "ALIPHATIC"):
                lbAA2.insert(END, "ALA");
                lbAA2.insert(END, "GLY"); 
                lbAA2.insert(END, "ILE");
                lbAA2.insert(END, "LEU");  
                lbAA2.insert(END, "MET");
                lbAA2.insert(END, "PRO");
                lbAA2.insert(END, "VAL");
                      
            elif (aa == "CHARGED"):
                lbAA2.insert(END, "ARG");
                lbAA2.insert(END, "HIS");   
                lbAA2.insert(END, "LYS");
                lbAA2.insert(END, "ASP"); 
                lbAA2.insert(END, "GLU");
                
            elif (aa == "POSITIVE"):
                lbAA2.insert(END, "ARG");
                lbAA2.insert(END, "HIS");   
                lbAA2.insert(END, "LYS");
                     
            elif (aa == "NEGATIVE"):
                lbAA2.insert(END, "ASP");
                lbAA2.insert(END, "GLU");  
                      
            elif (aa == "SULPHURCONT"):
                lbAA2.insert(END, "MET");
                lbAA2.insert(END, "CYS");     
                
            elif (aa == "TINY"):
                lbAA2.insert(END, "ALA");
                lbAA2.insert(END, "CYS"); 
                lbAA2.insert(END, "GLY");
                lbAA2.insert(END, "SER"); 
                
            elif (aa == "SMALL"):
                lbAA2.insert(END, "ASN");
                lbAA2.insert(END, "ASP"); 
                lbAA2.insert(END, "PRO");
                lbAA2.insert(END, "THR");
                lbAA2.insert(END, "VAL"); 
                
            elif (aa == "LARGE"):
                lbAA2.insert(END, "ARG");
                lbAA2.insert(END, "GLN");       
                lbAA2.insert(END, "GLU");
                lbAA2.insert(END, "HIS");
                lbAA2.insert(END, "ILE");
                lbAA2.insert(END, "LEU");
                lbAA2.insert(END, "LYS");
                lbAA2.insert(END, "MET");
                lbAA2.insert(END, "PHE");
                lbAA2.insert(END, "TRP");
                lbAA2.insert(END, "TYR");
            else:                             
                lbAA2.insert(END, aa)
            
       
       
    def __drawASAFilter(self):
        asaGUI = asagui.AccSurfDialog(asagui.MUTANT_MODE, self, self.selOSL)
   
    def addASAselection(self, selection, selOSL):
        if selOSL == self.selOSL:        
            filterRes =[]
            mol = chimera.selection.OSLSelection(self.selOSL)
            mol = mol.molecules()[0]
            res = mol.residues
           
            for r in res:
                if selection.count(r) > 0:
                    filterRes.append(r)
                   
            name = 'ASA'
            if self.filter[self.selOSL].has_key(name):
                i = 2
                while self.filter[self.selOSL].has_key(name + " " + str(i)):
                    i += 1
                   
                name = name + " " + str(i)
               
            self.__addFilterLine(name, filterRes)
        else:
            pass #TODO
 
   
    def __drawSelectionFilter(self):
#        invertVar = BooleanVar()
#        invertVar.set(False)
       
        fSelFilter = Frame(self.fFilterDetail, bg="#d3ebf7")
        fSelFilter.pack(side=TOP, fill=BOTH, expand=NO)
       
        self.filterElements.append(fSelFilter)
       
        strName = StringVar()
        strName.set('Chimera selection')
        lName = Label(fSelFilter, text='Name ')
        eName = Entry(fSelFilter, textvariable=strName, width=35, state=NORMAL)
       
        lName.grid(row=0, column=0, sticky=W, padx=5, pady=5)
        eName.grid(row=0, column=1, sticky=W, padx=5, pady=5, columnspan=5)
 
#        cbInvert = Checkbutton(fSelFilter, text = "Take inverted Chimera selection", state=NORMAL, anchor=W, variable=invertVar)
#        
#        cbInvert.grid(row=1, column=0, sticky=W, padx=5, pady=5)
 
        def addFilter(event=None):
            filterRes =[]
            mol = chimera.selection.OSLSelection(self.selOSL)
            mol = mol.molecules()[0]
            res = mol.residues
           
            selRes = chimera.selection.currentResidues()
           
            for r in res:
                if selRes.count(r) > 0:
                    filterRes.append(r)
               
            name = strName.get()
            if self.filter[self.selOSL].has_key(name):
                i = 2
                while self.filter[self.selOSL].has_key(name + " " + str(i)):
                    i += 1
                   
                name = name + " " + str(i)
               
            self.__addFilterLine(name, filterRes)
 
        btnAddFilter = Button(fSelFilter, text='Import selection as filter', command=addFilter, state=NORMAL)
        btnAddFilter.grid(row=10, column=10, sticky=W, padx=10, pady=10)
 
   
    def __drawSSFilter(self):
        helixVar = BooleanVar()
        helixVar.set(False)
        strandVar = BooleanVar()
        strandVar.set(False)
        otherVar = BooleanVar()
        otherVar.set(False)
       
        fSsFilter = Frame(self.fFilterDetail, bg="#d3ebf7")
        fSsFilter.pack(side=TOP, fill=BOTH, expand=NO)
       
        self.filterElements.append(fSsFilter)
       
        strName = StringVar()
        strName.set('Sec. Structure')
        lName = Label(fSsFilter, text='Name ')
        eName = Entry(fSsFilter, textvariable=strName, width=35, state=NORMAL)
       
        lName.grid(row=0, column=0, sticky=W, padx=5, pady=5)
        eName.grid(row=0, column=1, sticky=W, padx=5, pady=5, columnspan=5)
               
        lSecStr = Label(fSsFilter, text='Secondary Structure: ')
        cbHelix = Checkbutton(fSsFilter, text='helix', state=NORMAL, anchor=W, variable=helixVar)
        cbStrand = Checkbutton(fSsFilter, text='strand', state=NORMAL, anchor=W, variable=strandVar)
        cbOther = Checkbutton(fSsFilter, text='other', state=NORMAL, anchor=W, variable=otherVar)
 
        lSecStr.grid(row=1, column=0, sticky=W, padx=5, pady=5)
        cbHelix.grid(row=1, column=1, sticky=W, padx=5, pady=5)
        cbStrand.grid(row=1, column=2, sticky=W, padx=5, pady=5)
        cbOther.grid(row=1, column=3, sticky=W, padx=5, pady=5)
       
        def addFilter(event=None):
            filterRes =[]
            mol = chimera.selection.OSLSelection(self.selOSL)
            mol = mol.molecules()[0]
            res = mol.residues
           
            for r in res:
                if r.isHelix and helixVar.get():
                    filterRes.append(r)
                elif r.isStrand and strandVar.get():
                    filterRes.append(r)
                elif otherVar.get() and not r.isStrand and not r.isHelix:
                    filterRes.append(r)
                   
            name = strName.get()
            if self.filter[self.selOSL].has_key(name):
                i = 2
                while self.filter[self.selOSL].has_key(name + " " + str(i)):
                    i += 1
                   
                name = name + " " + str(i)
               
            self.__addFilterLine(name, filterRes)
 
        btnAddFilter = Button(fSsFilter, text='Add filter', command=addFilter, state=NORMAL)
        btnAddFilter.grid(row=10, column=10, sticky=W, padx=10, pady=10)
 
       
    def __drawUserFilter(self):
        fUserFilterName = Frame(self.fFilterDetail, bg="#d3ebf7")
        fUserFilterName.pack(side=TOP, fill=BOTH, expand=NO)
        self.filterElements.append(fUserFilterName)
       
        strName = StringVar()
        strName.set('User defined')
        lName = Label(fUserFilterName, text='Name ')
        eName = Entry(fUserFilterName, textvariable=strName, width=35, state=NORMAL)
       
        lName.grid(row=0, column=0, sticky=W, padx=5, pady=5)
        eName.grid(row=0, column=1, sticky=W, padx=5, pady=5)
 
        fUserFilter = Frame(self.fFilterDetail, bg="#d3ebf7")
        fUserFilter.pack(side=TOP, fill=BOTH, expand=YES)
        self.filterElements.append(fUserFilter)
       
        cPlotFilter = Canvas(fUserFilter, width=DEFAULT_PLOT_WIDTH, height=3*LETTER_HEIGHT, bg='gray')
        #cPlotFilter.pack(side=LEFT, anchor='w', expand=NO)
        cPlotFilter.grid(row=1, column=0, sticky=W+E, columnspan=3)
       
        sbarX = Scrollbar(fUserFilter, orient=HORIZONTAL)
        sbarX.config(command=cPlotFilter.xview)
        cPlotFilter.config(xscrollcommand=sbarX.set)
        #sbarX.pack(side=TOP, fill=X)
        sbarX.grid(row=2, column=0, sticky=W+E, columnspan=3)
       
        #draw seq.
        y = 2 * LETTER_HEIGHT
 
        mol = chimera.selection.OSLSelection(self.selOSL)
        mol = mol.molecules()[0]
        seqs = mol.sequences()
 
        i = 0
 
        objs = {}
        resis = {}
 
        for s in seqs:
            i += 1
            x = DESCR_SPACE + i * LETTER_WIDTH
           
            chainLetter = cPlotFilter.create_text(x, y, text='|', font=FONT)            
            objs[chainLetter] = 'start of chain ' + s.chain
            chainLetter = cPlotFilter.create_text(x, y - LETTER_HEIGHT, text=s.chain, font=FONT)
            objs[chainLetter] = 'start of chain ' + s.chain
                       
            for r in s.residues:
                if r == None:
                    continue
                i += 1
                l1 = protein3to1[r.type]
                x = DESCR_SPACE + i * LETTER_WIDTH
                resLetter = cPlotFilter.create_text(x, y, text=l1, font=FONT)
                objs[resLetter] = r.oslIdent()
                resis[resLetter] = {'res':r, 'x':x, 'y':y, 'rect':None}
       
        cPlotFilter.config(scrollregion=(0, 0, (i+1) * LETTER_WIDTH + DESCR_SPACE, 3 * LETTER_HEIGHT))
       
       
        #add filter methods        
        self.canvasLegendUF = None
       
        def __canvasMotionUF(event):
            obj = event.widget.find_withtag(CURRENT)
       
            cPlotFilter.delete(self.canvasLegendUF)
       
            if len(obj) < 1:# and not self.canvasLegend == None:
                return
 
            if objs.has_key(obj[0]):
                self.canvasLegendUF = cPlotFilter.create_text(cPlotFilter.canvasx(event.x), \
                                                       cPlotFilter.canvasy(event.y) - 15, \
                                                       text=objs[obj[0]], \
                                                       anchor=S, fill='blue')
               
        cPlotFilter.bind('<Motion>', __canvasMotionUF)
       
        selectedRes = []
        selRect = None
        selRectX1 = 0
        selRectX2 = 0
        selRectY1 = 0
        selRectY2 = 0
       
        def __canvasClickUF(event):
            obj = event.widget.find_withtag(CURRENT)
       
            global selRect, selRectX1, selRectX2, selRectY1, selRectY2
       
            selRectX1 = cPlotFilter.canvasx(event.x)
            selRectX2 = selRectX1
            selRectY1 = cPlotFilter.canvasy(event.y)
            selRectY2 = selRectY1
               
            selRect = cPlotFilter.create_rectangle(selRectX1, selRectY1, selRectX2, \
                                                   selRectY2, outline='green', width=1, dash=(2, 2))
           
            if len(obj) < 1:
                return
             
            if resis.has_key(obj[0]):
                res = resis[obj[0]]['res']
                if res in selectedRes:
                    cPlotFilter.delete(resis[obj[0]]['rect'])
                    resis[obj[0]]['rect'] = None
                    selectedRes.remove(res)
                else:
                    x1 = resis[obj[0]]['x'] - LETTER_WIDTH/2
                    x2 = x1 + LETTER_WIDTH
                    y1 = resis[obj[0]]['y'] - LETTER_HEIGHT/2
                    y2 = y1 + LETTER_HEIGHT
                    resis[obj[0]]['rect'] = cPlotFilter.create_rectangle(x1, y1, x2, y2, outline=FILTER_COLOR, width=2, fill=None)
                    selectedRes.append(resis[obj[0]]['res'])
 
        cPlotFilter.bind('<Button-1>', __canvasClickUF)
       
        def __canvasB1MotionUF(event):
            try:
                global selRect, selRectX1, selRectX2, selRectY1, selRectY2
               
                selRectX2 = event.widget.canvasx(event.x)
                selRectY2 = event.widget.canvasy(event.y)
                cPlotFilter.coords(selRect, selRectX1, selRectY1, event.widget.canvasx(event.x), event.widget.canvasy(event.y))
            except: return
       
        cPlotFilter.bind("<B1-Motion>", __canvasB1MotionUF)
       
        def __canvasB1ReleaseUF(event):
            try:
                __canvasB1MotionUF(event)
                global selRect, selRectX1, selRectX2, selRectY1, selRectY2
               
                cPlotFilter.delete(selRect)
               
                if selRectY1 > selRectY2:
                    ycurr = selRectY2
                    selRectY2 = selRectY1
                    selRectY1 = ycurr
                   
                if selRectX1 > selRectX2:
                    xcurr = selRectX2
                    selRectX2 = selRectX1
                    selRectX1 = xcurr
               
                if selRectY1 < 2.5 * LETTER_HEIGHT and selRectY2 > 1.5 * LETTER_HEIGHT:
                    for o in resis.keys():
                        if resis[o]['x'] > selRectX1 and resis[o]['x'] < selRectX2:
                            res = resis[o]['res']
                            if not res in selectedRes:
                                x1 = resis[o]['x'] - LETTER_WIDTH/2
                                x2 = x1 + LETTER_WIDTH
                                y1 = resis[o]['y'] - LETTER_HEIGHT/2
                                y2 = y1 + LETTER_HEIGHT
                                resis[o]['rect'] = cPlotFilter.create_rectangle(x1, y1, x2, y2, outline=FILTER_COLOR, width=2, fill=None)
                                selectedRes.append(resis[o]['res'])
            except: return        
       
        cPlotFilter.bind("<ButtonRelease-1>", __canvasB1ReleaseUF)        
       
        #buttons
        def addFilter(event=None):
            name = strName.get()
            if self.filter[self.selOSL].has_key(name):
                i = 2
                while self.filter[self.selOSL].has_key(name + " " + str(i)):
                    i += 1
                   
                name = name + " " + str(i)
               
            self.__addFilterLine(name, selectedRes)
       
        btnAddFilter = Button(fUserFilter, text='Add filter', command=addFilter, state=NORMAL)
        btnAddFilter.grid(row=10, column=2, sticky=E, padx=10, pady=10)
   
   
   
    #**********************************************************************************
    #draw GUI
    #**********************************************************************************
   
    def __drawOptionFrame(self):
        lModel = Label(self.fOptions, text='Model')    
        self.cbModels = ComboBox(self.fOptions, listbox_width=25, dropdown=1, entry_width=25, \
                                 selectioncommand=self.__selectModel, scrolledlist_items=[])
        lModel.grid(row=0, column=0, sticky=W, padx=0, pady=5)
       
        if self.mode == DEFAULT_MODE:
            self.cbModels.grid(row=0, column=1, sticky=W, padx=5, pady=5)
        else:
            self.lModel = Label(self.fOptions, text=self.selOSL)
            self.lModel.grid(row=0, column=1, sticky=W, padx=5, pady=5)
           
        Label(self.fOptions, text="\t\t\t\t ").grid(row=0, column=2, columnspan=2, sticky=W, padx=5, pady=5) #placeholder      
       
        sep = Frame(self.fOptions, height=2, bd=1, relief=SUNKEN, bg="#d3ebf7")
        sep.grid(row=1, column=0, sticky = W+E, padx = 0, pady=6, columnspan=3)
       
       
        fOperator = Frame(self.fOptions, bg="#d3ebf7")
        fOperator.grid(row=2, column=0, sticky=W, columnspan=3)
       
        Label(fOperator, text='Operator: ').pack(side=LEFT)
        Radiobutton(fOperator, text='AND', value="AND", variable=self.operatorVar, command=self.__selectOperator).pack(side=LEFT)
        Radiobutton(fOperator, text='OR', value="OR", variable=self.operatorVar, command=self.__selectOperator).pack(side=LEFT)
 
   
    def __drawMaskFrame(self):
        fFilters = Frame(self.fMask, bg="#d3ebf7")
        fFilters.pack(side=TOP, fill=X, expand=YES)
        self.cPlot = Canvas(fFilters, width=DEFAULT_PLOT_WIDTH, height=DEFAULT_PLOT_HEIGHT, bg='gray')
        self.cPlot.pack(side=LEFT, anchor='w', expand=YES)
        self.cPlot.bind('<Motion>', self.__canvasMotion)
        self.cPlot.bind('<Button-1>', self.__canvasClick)
       
        self.canvasLegend = None
       
        sbarX = Scrollbar(self.fMask, orient=HORIZONTAL)
        sbarX.config(command=self.cPlot.xview)
        self.cPlot.config(xscrollcommand=sbarX.set)
        sbarX.pack(side=TOP, fill=X)
           
        sbarY = Scrollbar(fFilters, orient=VERTICAL)
        sbarY.config(command=self.cPlot.yview)
        self.cPlot.config(yscrollcommand=sbarY.set)
        sbarY.pack(side=RIGHT, fill=Y)
 
       
   
    def __drawFilterFrame(self):
        Label(self.fFilter, text='Add filter').pack(side=LEFT, anchor='w')
       
        filter = self.filterTypes.keys()
        filter.sort()
       
        self.filterElements =[]
       
        self.cbFilterType = ComboBox(self.fFilter, listbox_width=30, dropdown=1, entry_width=25, \
                                 selectioncommand=self.__selectFilter, scrolledlist_items=filter)
   
        self.cbFilterType.selectitem('')
        self.lastFilterMethod=''
        self.cbFilterType.pack(side=LEFT, anchor='w')
       
   
    def __drawButtonFrame(self):
        self.btnSelect = Button(self.fButton, text='Select', command=self.__selectResidues, state=NORMAL)
        self.btnSelect.pack(side=LEFT, anchor='w')
       
        if self.mode == MUTANT_MODE:
            self.btnReturnMask = Button(self.fButton, text='Return Mask', command=self.__returnMask, state=NORMAL)
            self.btnReturnMask.pack(side=LEFT, anchor='w')
 
    #init GUI
    #========
 
    def fillInUI(self, parent):
        self.parent = parent
        self.fMain = Frame(self.parent, bg="#d3ebf7")
        self.fMain.pack(fill=BOTH, expand=YES)
       
        self.fOptions = Frame(self.fMain, bg="#d3ebf7")
        self.fMask = Frame(self.fMain, bg="#d3ebf7")
        self.fFilter = Frame(self.fMain, bg="#d3ebf7")
        self.fFilterDetail = Frame(self.fMain, bg="#d3ebf7")
        self.fButton = Frame(self.fMain, bg="#d3ebf7")
       
        self.fOptions.pack(side=TOP, fill=X, expand=NO)
        self.fMask.pack(side=TOP, fill=BOTH, expand=YES)
        self.fFilter.pack(side=TOP, fill=X, expand=NO)
        self.fFilterDetail.pack(side=TOP, fill=X, expand=NO)
        sep = Frame(self.fMain, height=2, bd=1, bg="#d3ebf7", relief=SUNKEN)
        sep.pack(side=TOP, fill=X, expand=YES)
        self.fButton.pack(side=TOP, fill=X, expand=NO)
       
        self.__drawOptionFrame()
        self.__drawMaskFrame()
        self.__drawFilterFrame()
        self.__drawButtonFrame()
       
 
       
chimera.dialogs.register(MaskBuilderDialog.name, MaskBuilderDialog)
